using System;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using Ato.EN.IntegrationServices.Contrib.VaTS;
using Ato.EN.IntegrationServices.Document.DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTRT;
using Ato.EN.IntegrationServices.CodeGenerationFTER;

namespace Ato.CD.Inbound.TRT202202
{
    internal class CrossFormValidatorFTER : ICrossFormValidator
    {
        private TRT2022 ParentDocument { get; }

        private FTER2021 ChildDocument { get; }

        internal CrossFormValidatorFTER(TRT2022 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (FTER2021)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOFTER408001(response);
            VRATOFTER408029(response);
            VRATOFTER408033(response);
            VRATOFTER408112(response);
            VRATOFTER408162(response);
            VRATOFTER408205(response);
            VRATOFTER408371(response);
            VRATOFTER408372(response);

            return response;
        }

        #region VR.ATO.FTER.408001

        /*  VR.ATO.FTER.408001
        TFN on Schedule must match TFN on main form

        Legacy Rule Format:
        WHERE PARENT RETURN EXISTS 
        IF (FTER:RP:entity.identifier.TFN <> NULLORBLANK) AND (FTER:RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
            RETURN VALIDATION MESSAGE
        ENDIF

        Technical Business Rule Format:
        (^TFN <> BLANK) AND (^TFN <> ^ParentTFN)

        Data Elements:
            
        ^ParentTFN = ParentTFN
            
        ^TFN = RP
        */
        private void VRATOFTER408001(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER3) && !string.IsNullOrWhiteSpace(ParentDocument.RPIdentifierTFN) &&  !ChildDocument.FTER3.Equals(ParentDocument.RPIdentifierTFN);

            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN on Schedule must match TFN on main form",
                    Location = "/tns:FTER/tns:RP/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408001" } }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408001

        #region VR.ATO.FTER.408029

        /*  VR.ATO.FTER.408029
        Income year for variation must match the year of return on the main form

        Legacy Rule Format:
        WHERE PARENT RETURN EXISTS
        IF [FTER95] = "V" AND [FTER40] <> NULL AND [FTER40] <> PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year
            RETURN VALIDATION MESSAGE
        ENDIF
        [FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        [FTER40] = FTER:RP:Elections.FamilyTrustElectionStatus.Year

        Technical Business Rule Format:
        ^FTER95 = 'V' AND ^FTER40 <> NULL AND ^FTER40 <> ^ParentTargetFinancialYear

        Data Elements:
            
        RP:^FTER40 = tns:ElectionsFamilyTrustElectionStatusY
            
        RP:^FTER95 = tns:ElectionsFamilyTrustElectionRevocationC
            
        RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
        */
        private void VRATOFTER408029(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.FTER95 == @"V" &&  ChildDocument.FTER40.HasValue  && ! ChildDocument.FTER40.GetValueOrDefault().Equals (ParentDocument.TRT1.GetValueOrDefault ());

            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408029",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year for variation must match the year of return on the main form",
                    LongDescription = @"Income year specified for the purpose of the FTE variation must match the year of return on the main form",
                    Location = "/tns:FTER/tns:RP/tns:ElectionOrVariationIncomeYear/tns:ElectionsFamilyTrustElectionStatusY",
                    Parameters = new ProcessMessageParameters()
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408029" },
                        new ProcessMessageParameter { Name = "FTER95", Value = ChildDocument.FTER95 },
                        new ProcessMessageParameter { Name = "FTER40", Value = ChildDocument.FTER40.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408029

        #region VR.ATO.FTER.408033

        /*  VR.ATO.FTER.408033
        Revocation indicated and income year of revocation not the same as year of return

        Legacy Rule Format:
        IF [FTER95] = "R" AND [FTER96] <> NULL AND [FTER96] <> PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year
            RETURN VALIDATION MESSAGE
        ENDIF
        [FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        [FTER96] = FTER:RP:Elections.FamilyTrustRevocationStatusEffective.Year

        Technical Business Rule Format:
        ^FTER95 = 'R' AND ^FTER96 <> NULL AND ^FTER96 <> ^ParentTargetFinancialYear

        Data Elements:
            
        RP:^FTER96 = tns:ElectionsStatusEffectiveY
            
        RP:^FTER95 = tns:ElectionsFamilyTrustElectionRevocationC
            
        RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
        */
        private void VRATOFTER408033(List<ProcessMessageDocument> response)
        {
            bool assertion =  ChildDocument.FTER95 == @"R" && ChildDocument.FTER96.HasValue && !ChildDocument.FTER96.GetValueOrDefault().Equals(ParentDocument.TRT1.GetValueOrDefault ());

            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408390",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year of revocation must be the same as the year of return on the main form",
                    Location = "/tns:FTER/tns:RP/tns:FamilyTrustRevocation/tns:ElectionsStatusEffectiveY",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408033" },
                        new ProcessMessageParameter { Name = "FTER95", Value = ChildDocument.FTER95 },
                        new ProcessMessageParameter { Name = "FTER96", Value = ChildDocument.FTER96.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408033

        #region VR.ATO.FTER.408112

        /*  VR.ATO.FTER.408112
        Income year specified on main return form must equal the income year specified on the FTE schedule.

        Legacy Rule Format:
        WHERE PARENT RETURN EXISTS
        IF [FTER95] = "E" AND [FTER40] <>  PARENT RETURN:RP:Elections.FamilyTrustElectionStatus.Year
            RETURN VALIDATION MESSAGE
        ENDIF
        [FTER95] = FTER:RP:Elections.FamilyTrustElectionRevocation.Code
        [FTER40] = FTER:RP:Elections.FamilyTrustElectionStatus.Year

        Technical Business Rule Format:
        ^FTER95 = 'E' AND ^FTER40 <>  ^ParentFamilyTrustElectionStatusYear

        Data Elements:
            
        RP:^FTER40 = tns:ElectionsFamilyTrustElectionStatusY
            
        RP:^FTER95 = tns:ElectionsFamilyTrustElectionRevocationC
            
        RP:^ParentFamilyTrustElectionStatusYear = ParentFamilyTrustElectionStatusYear
        */
        private void VRATOFTER408112(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.FTER95 == @"E" && ChildDocument.FTER40.HasValue && !ChildDocument.FTER40.GetValueOrDefault().Equals(ParentDocument.TRT28.GetValueOrDefault ());

            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408377",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Income year specified on main return form must equal the income year specified on the schedule",
                    Location = "/tns:FTER/tns:RP/tns:ElectionOrVariationIncomeYear/tns:ElectionsFamilyTrustElectionStatusY",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408112" },
                        new ProcessMessageParameter { Name = "FTER95", Value = ChildDocument.FTER95 },
                        new ProcessMessageParameter { Name = "FTER40", Value = ChildDocument.FTER40.GetValueOrDefault().ToString() }
                    }
                };
                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408112

        #region VR.ATO.FTER.408162

        /*  VR.ATO.FTER.408162
        Revocation date must not be after 30 June of the year of the parent return

        Legacy Rule Format:
        WHERE PARENT RETURN EXISTS
        IF [FTER57] > ((PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year)&"-06-30")
            RETURN VALIDATION MESSAGE 
        ENDIF
        [FTER57] = FTER:RP:Elections.Revocation.Date

        Technical Business Rule Format:
        ^FTER57 > AsDate(Concat(^ParentTargetFinancialYear, '-06-30'))

        Data Elements:
            
        RP:^FTER57 = tns:Elections.Revocation.Date
            
        RP:^ParentTargetFinancialYear = ParentTargetFinancialYear
        */
        private void VRATOFTER408162(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.FTER57.GetValueOrDefault() > AsDate(string.Concat(ParentDocument.TRT1.GetValueOrDefault(), @"-06-30"));

            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408389",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Revocation date must not be after 30 June of the year of return on the main form",
                    Location = "/tns:FTER/tns:RP/tns:FamilyTrustRevocation/tns:ElectionsD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408162" },
                        new ProcessMessageParameter { Name = "FTER57", Value =  ChildDocument.FTER57.GetValueOrDefault().ToString ()}
                    }
                };

                 response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408162

        #region VR.ATO.FTER.408205

        /*  VR.ATO.FTER.408205
        Your supplied ABN does not match the ABN supplied on the form it was submitted with

        Legacy Rule Format:
        IF (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) AND (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
            RETURN VALIDATION MESSAGE
        ENDIF

        Technical Business Rule Format:
        (^FTER170 <> BLANK) AND (^ParentABN <> BLANK) AND (^FTER170 <> ^ParentABN)

        Data Elements:
            
        RP:^FTER170 = tns:Identifiers.AustralianBusinessNumber.Identifier
            
        RP:^ParentABN = ParentABN
        */
        private void VRATOFTER408205(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.FTER170)  
                        && !string.IsNullOrWhiteSpace(ParentDocument.TRT7) 
                        && !ChildDocument.FTER170.Equals(ParentDocument.TRT7);
            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/tns:FTER/tns:RP/tns:AustralianBusinessNumberId",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408205" },
                        new ProcessMessageParameter { Name = "FTER170", Value = ChildDocument.FTER170 }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402010 

        #region VR.ATO.FTER.408371

        /*  VR.ATO.FTER.408371
        Reporting period start date must match the period start date on the main form
        
        Technical Business Rule Format:
        WHERE PARENT RETURN EXISTS
        ^FTER173 <> ^ParentPeriodStartDate

        Data Elements:
            
        RP:^FTER173 = tns:Period.Start.Date
            
        RP:^ParentPeriodStartDate = ParentPeriodStartDate
        */
        private void VRATOFTER408371(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.RPStartDate.Value != ChildDocument.FTER173.Value;
            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408371",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting period start date must match the period start date on the main form",
                    Location = "/tns:FTER/tns:RP/tns:PeriodStartD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408371" },
                        new ProcessMessageParameter { Name = "FTER173", Value = ChildDocument.FTER173.Value.ToString("yyyy-MM-dd") }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408371 

        #region VR.ATO.FTER.408372

        /*  VR.ATO.FTER.408372
        Reporting period end date must match the period end date on the main form
        
        Technical Business Rule Format:
        WHERE PARENT RETURN EXISTS
        ^FTER174 <> ^ParentPeriodEndDate

        Data Elements:
            
        RP:^FTER174 = tns:Period.End.Date
            
        RP:^ParentPeriodEndDate = ParentPeriodEndDate
        */
        private void VRATOFTER408372(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.RPEndDate.Value.Equals(ChildDocument.FTER174.Value);
            if (assertion)
            {
                ProcessMessageDocument processMessage = new ProcessMessageDocument
                {
                    Code = "CMN.ATO.FTER.408372",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Reporting period end date must match the period end date on the main form",
                    Location = "/tns:FTER/tns:RP/tns:PeriodEndD",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.FTER.408372" },
                        new ProcessMessageParameter { Name = "FTER174", Value = ChildDocument.FTER174.Value.ToString("yyyy-MM-dd") }
                    }
                };

                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.FTER.408372 

        private DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = date;

            return response;
        }
    }
}