using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Text.RegularExpressions;
using Au.Gov.Sbr.Xbrl.Document;
using DataContracts;
using Au.Gov.Sbr.Xbrl.Metadata;
using System.Runtime.CompilerServices;

namespace Ato.EN.IntegrationServices.CodeGenerationIDS
{
    public class IDS2021Consumer
    {

        private static Regex StripWhitespace = new Regex(@"\s*");

        #region Helper/Validation Methods
        private static bool? ToBoolean(string str)
        {
            bool returnValue;
            string value = (str ?? "").Trim();
            if (value == "0" || value == "1")
                return Convert.ToBoolean(Convert.ToInt32(value));
            
            if (Boolean.TryParse(value, out returnValue))
                return returnValue;
            return null;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }
				
        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Derives the SBR element location.
        /// </summary>
        /// <param name="sbrElement">The SBR element.</param>
        /// <param name="addOccurrenceIndex">if set to <c>true</c> [add occurrence the occurrence index to the path].</param>
        /// <returns></returns>
        private static string DeriveSbrElementLocation(SBRElement sbrElement, bool addOccurrenceIndex = false)
        {
            if (sbrElement == null) return "/xbrli:xbrl";
            int xpathOccurrence = sbrElement.OccurrenceIndex + 1;
            string location;

            if (addOccurrenceIndex)
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name + "[" +
                           xpathOccurrence + "]";
            }
            else
            {
                location = "/" + sbrElement.NamespacePrefix + ":" + sbrElement.Name;
            }

            if (sbrElement.IsInTuple)
            {
                location = DeriveSbrElementLocation(sbrElement.ParentTuple, true) + location;
            }
            else
            {
                location = "/xbrli:xbrl" + location;
            }

            return location;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

         #endregion

        public IDS2021 Consume(SBRDocument sbrDocument)
        {
            List<ProcessMessageDocument> errors;
            return Consume(sbrDocument, false, out errors);
        }

        /// <summary>
        /// Allows for validation of rules that can not be generated and must run at a lower level.
        /// A sub-class will need to be created with an override for this method - which does nothing in its base implementation.
        /// </summary>
        /// <param name="sbrDocument">The SBR document.</param>
        /// <param name="populatedReportElementsMap">The populated report elements map can be used to find data elements and tuples.</param>
        /// <param name="contextArray">The context array can be used to find context values.</param>
        /// <param name="errors">The errors collection is to be populated with validation results.</param>
        protected virtual void ValidateUnsupportedRules(SBRDocument sbrDocument, IDictionary<string, SBRElement> populatedReportElementsMap, SBRContext[] contextArray, List<ProcessMessageDocument> errors)
        {

        }

        public IDS2021 Consume(SBRDocument sbrDocument, bool validationMode, out List<ProcessMessageDocument> errors)
        {
            IDS2021 report;
            report = new IDS2021();
            errors = new List<ProcessMessageDocument>();

            SBRContext[] contextArray;
            IDictionary<string, SBRElement> populatedReportElementsMap;

            contextArray = sbrDocument.ContextsMap.Values.ToArray();
            populatedReportElementsMap = sbrDocument.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();

            if (validationMode) ValidateUnsupportedRules(sbrDocument, populatedReportElementsMap, contextArray, errors);

            /// <summary>
            /// Current element being extracted
            /// </summary>
            SBRElement currentValue = null;
            /// <summary>
            /// Current elements being extracted
            /// </summary>
            IList<SBRElement> currentValues = null;
            /// <summary>
            /// Current occurrences being extracted
            /// </summary>
            IList<SBRElement> currentOccurrences = null;
            /// <summary>
            /// Working Variable for date time variables
            /// </summary>
            DateTime currentDateTimeValue;
            /// <summary>
            /// Working Variable for boolean variables
            /// </summary>
            bool currentBooleanValue;
            /// <summary>
            /// Working Variable for decimal variables
            /// </summary>
            decimal currentDecimalValue;
            /// <summary>
            /// Working Variable for integer variables
            /// </summary>
            int currentIntValue;
            /// <summary>
            /// Working Variable for long variables
            /// </summary>
            long currentLongValue;
            XmlDocumentFragment currentSegment;
            XmlNamespaceManager currentNamespaceManager;
            XmlNode currentNode;
    
            #region sbrRPContext
            IEnumerable<SBRContext> sbrRPContexts = null;
            SBRContext sbrRPContext = null;
    
            sbrRPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 1 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty")) )).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPContexts != null && sbrRPContexts.Count() > 0)
            {
                sbrRPContext = sbrRPContexts.First();
        
                report.RPId = sbrRPContext.Id;
                report.RPIdentifier = sbrRPContext.EntityIdentifier;
                report.RPIdentifierScheme = sbrRPContext.EntityScheme;
        
                report.RPCount = sbrRPContexts.Count();
                report.RPExists = true;
                report.RPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContext) + 1;
                report.RPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPContexts.Last()) + 1;
        

                if (report.RPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPIdentifierTFN = report.RPIdentifier;
    
                report.RPStartDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationStartAsDateObject;
                report.RPEndDate = ((SBRDurationPeriod)sbrRPContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPContexts.Count(); i++)
                {
                    sbrRPContext = sbrRPContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.CountryByCountryReportApplicability.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS330 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.LocalFilePartAVoluntaryLodgment.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS331 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS1 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.SmallBusinessConcessionsEligibility.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS425 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.RelatedPartiesTransactions.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS2 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrForeignCountriesCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}ForeignCountries", out currentValue))
                        sbrForeignCountriesCollection = currentValue.Occurrences;
                    else
                        sbrForeignCountriesCollection = null;
            
                    #region sbrForeignCountriesCollection
            
                    if (sbrForeignCountriesCollection != null)
                    {
                        report.ForeignCountriesCollectionExists = true;
                        report.ForeignCountriesCollectionCount = sbrForeignCountriesCollection.Count();
                        if (sbrForeignCountriesCollection != null && sbrForeignCountriesCollection.Count() > 0)
                        {
                            report.ForeignCountriesCollection = new List<IDS2021.ForeignCountries>();
                            for (int tupleIndex = 0; tupleIndex < sbrForeignCountriesCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrForeignCountries;
                                sbrForeignCountries = sbrForeignCountriesCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrForeignCountriesTupleElementMap;
                                sbrForeignCountriesTupleElementMap = sbrForeignCountries.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.ForeignCountries foreignCountries = new IDS2021.ForeignCountries();
                                report.ForeignCountriesCollection.Add(foreignCountries);
                                foreignCountries.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrForeignCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) foreignCountries.IDS418 = currentValue.Value;
                                }
                    
                                IEnumerable<SBRElement> sbrForeignCountries_ActivityCodeCollection;
                                if (sbrForeignCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}ActivityCode", out currentValue))
                                    sbrForeignCountries_ActivityCodeCollection = currentValue.Occurrences;
                                else
                                    sbrForeignCountries_ActivityCodeCollection = null;
                        
                                #region sbrForeignCountries_ActivityCodeCollection
                        
                                if (sbrForeignCountries_ActivityCodeCollection != null)
                                {
                                    foreignCountries.ForeignCountries_ActivityCodeCollectionExists = true;
                                    foreignCountries.ForeignCountries_ActivityCodeCollectionCount = sbrForeignCountries_ActivityCodeCollection.Count();
                                    if (sbrForeignCountries_ActivityCodeCollection != null && sbrForeignCountries_ActivityCodeCollection.Count() > 0)
                                    {
                                        foreignCountries.ForeignCountries_ActivityCodeCollection = new List<IDS2021.ForeignCountries.ForeignCountries_ActivityCode>();
                                        for (int tupleIndex1 = 0; tupleIndex1 < sbrForeignCountries_ActivityCodeCollection.Count(); tupleIndex1++)
                                        {
                                            SBRElement sbrForeignCountries_ActivityCode;
                                            sbrForeignCountries_ActivityCode = sbrForeignCountries_ActivityCodeCollection.ElementAt(tupleIndex1);
                            
                                            IDictionary<string, SBRElement> sbrForeignCountries_ActivityCodeTupleElementMap;
                                            sbrForeignCountries_ActivityCodeTupleElementMap = sbrForeignCountries_ActivityCode.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                            IDS2021.ForeignCountries.ForeignCountries_ActivityCode activityCode = new IDS2021.ForeignCountries.ForeignCountries_ActivityCode();
                                            foreignCountries.ForeignCountries_ActivityCodeCollection.Add(activityCode);
                                            activityCode.OccurrenceIndex = tupleIndex1 + 1;
                                    
                                            if (sbrForeignCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountryActivity.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS419 = currentValue.Value;
                                            }
                                    
                                            if (sbrForeignCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS5 = decimal.Parse(currentValue.Value);
                                            }
                                    
                                            if (sbrForeignCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS6 = decimal.Parse(currentValue.Value);
                                            }
                                } 
                                #endregion End of Tuple sbrForeignCountries_ActivityCodeCollection
                                    } 
                                } 
                        
                                if (sbrForeignCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Other.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) foreignCountries.IDS7 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrForeignCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Other.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) foreignCountries.IDS8 = decimal.Parse(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrForeignCountriesCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.SpecifiedCountriesTransactions.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS9 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrSpecifiedCountriesCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}SpecifiedCountries", out currentValue))
                        sbrSpecifiedCountriesCollection = currentValue.Occurrences;
                    else
                        sbrSpecifiedCountriesCollection = null;
            
                    #region sbrSpecifiedCountriesCollection
            
                    if (sbrSpecifiedCountriesCollection != null)
                    {
                        report.SpecifiedCountriesCollectionExists = true;
                        report.SpecifiedCountriesCollectionCount = sbrSpecifiedCountriesCollection.Count();
                        if (sbrSpecifiedCountriesCollection != null && sbrSpecifiedCountriesCollection.Count() > 0)
                        {
                            report.SpecifiedCountriesCollection = new List<IDS2021.SpecifiedCountries>();
                            for (int tupleIndex = 0; tupleIndex < sbrSpecifiedCountriesCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrSpecifiedCountries;
                                sbrSpecifiedCountries = sbrSpecifiedCountriesCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrSpecifiedCountriesTupleElementMap;
                                sbrSpecifiedCountriesTupleElementMap = sbrSpecifiedCountries.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.SpecifiedCountries specifiedCountries = new IDS2021.SpecifiedCountries();
                                report.SpecifiedCountriesCollection.Add(specifiedCountries);
                                specifiedCountries.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrSpecifiedCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) specifiedCountries.IDS420 = currentValue.Value;
                                }
                    
                                IEnumerable<SBRElement> sbrSpecifiedCountries_ActivityCodeCollection;
                                if (sbrSpecifiedCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}ActivityCode", out currentValue))
                                    sbrSpecifiedCountries_ActivityCodeCollection = currentValue.Occurrences;
                                else
                                    sbrSpecifiedCountries_ActivityCodeCollection = null;
                        
                                #region sbrSpecifiedCountries_ActivityCodeCollection
                        
                                if (sbrSpecifiedCountries_ActivityCodeCollection != null)
                                {
                                    specifiedCountries.SpecifiedCountries_ActivityCodeCollectionExists = true;
                                    specifiedCountries.SpecifiedCountries_ActivityCodeCollectionCount = sbrSpecifiedCountries_ActivityCodeCollection.Count();
                                    if (sbrSpecifiedCountries_ActivityCodeCollection != null && sbrSpecifiedCountries_ActivityCodeCollection.Count() > 0)
                                    {
                                        specifiedCountries.SpecifiedCountries_ActivityCodeCollection = new List<IDS2021.SpecifiedCountries.SpecifiedCountries_ActivityCode>();
                                        for (int tupleIndex1 = 0; tupleIndex1 < sbrSpecifiedCountries_ActivityCodeCollection.Count(); tupleIndex1++)
                                        {
                                            SBRElement sbrSpecifiedCountries_ActivityCode;
                                            sbrSpecifiedCountries_ActivityCode = sbrSpecifiedCountries_ActivityCodeCollection.ElementAt(tupleIndex1);
                            
                                            IDictionary<string, SBRElement> sbrSpecifiedCountries_ActivityCodeTupleElementMap;
                                            sbrSpecifiedCountries_ActivityCodeTupleElementMap = sbrSpecifiedCountries_ActivityCode.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                            IDS2021.SpecifiedCountries.SpecifiedCountries_ActivityCode activityCode = new IDS2021.SpecifiedCountries.SpecifiedCountries_ActivityCode();
                                            specifiedCountries.SpecifiedCountries_ActivityCodeCollection.Add(activityCode);
                                            activityCode.OccurrenceIndex = tupleIndex1 + 1;
                                    
                                            if (sbrSpecifiedCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountryActivity.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS421 = currentValue.Value;
                                            }
                                    
                                            if (sbrSpecifiedCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS12 = decimal.Parse(currentValue.Value);
                                            }
                                    
                                            if (sbrSpecifiedCountries_ActivityCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) activityCode.IDS13 = decimal.Parse(currentValue.Value);
                                            }
                                } 
                                #endregion End of Tuple sbrSpecifiedCountries_ActivityCodeCollection
                                    } 
                                } 
                        
                                if (sbrSpecifiedCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Other.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) specifiedCountries.IDS14 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrSpecifiedCountriesTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Other.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) specifiedCountries.IDS15 = decimal.Parse(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrSpecifiedCountriesCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TransactionsTangiblePropertyRevenue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS16 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TransactionsRoyaltiesLicenceFees.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS21 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TransactionsRentLeasing.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS30 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TransactionsServiceArrangements.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS35 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.DerivativeTransactions.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS81 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.DerivativesTradingGlobal.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS87 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.DebtFactoringSecuritisation.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS88 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.OtherFinancialDealings.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS97 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeGainLoss.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS333 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrTransactionTypeCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}TransactionType", out currentValue))
                        sbrTransactionTypeCollection = currentValue.Occurrences;
                    else
                        sbrTransactionTypeCollection = null;
            
                    #region sbrTransactionTypeCollection
            
                    if (sbrTransactionTypeCollection != null)
                    {
                        report.TransactionTypeCollectionExists = true;
                        report.TransactionTypeCollectionCount = sbrTransactionTypeCollection.Count();
                        if (sbrTransactionTypeCollection != null && sbrTransactionTypeCollection.Count() > 0)
                        {
                            report.TransactionTypeCollection = new List<IDS2021.TransactionType>();
                            for (int tupleIndex = 0; tupleIndex < sbrTransactionTypeCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrTransactionType;
                                sbrTransactionType = sbrTransactionTypeCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrTransactionTypeTupleElementMap;
                                sbrTransactionTypeTupleElementMap = sbrTransactionType.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.TransactionType transactionType = new IDS2021.TransactionType();
                                report.TransactionTypeCollection.Add(transactionType);
                                transactionType.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrTransactionTypeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeTransactionType.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) transactionType.IDS422 = currentValue.Value;
                                }
                    
                                IEnumerable<SBRElement> sbrTransactionType_CurrencyCodeCollection;
                                if (sbrTransactionTypeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}CurrencyCode", out currentValue))
                                    sbrTransactionType_CurrencyCodeCollection = currentValue.Occurrences;
                                else
                                    sbrTransactionType_CurrencyCodeCollection = null;
                        
                                #region sbrTransactionType_CurrencyCodeCollection
                        
                                if (sbrTransactionType_CurrencyCodeCollection != null)
                                {
                                    transactionType.TransactionType_CurrencyCodeCollectionExists = true;
                                    transactionType.TransactionType_CurrencyCodeCollectionCount = sbrTransactionType_CurrencyCodeCollection.Count();
                                    if (sbrTransactionType_CurrencyCodeCollection != null && sbrTransactionType_CurrencyCodeCollection.Count() > 0)
                                    {
                                        transactionType.TransactionType_CurrencyCodeCollection = new List<IDS2021.TransactionType.TransactionType_CurrencyCode>();
                                        for (int tupleIndex1 = 0; tupleIndex1 < sbrTransactionType_CurrencyCodeCollection.Count(); tupleIndex1++)
                                        {
                                            SBRElement sbrTransactionType_CurrencyCode;
                                            sbrTransactionType_CurrencyCode = sbrTransactionType_CurrencyCodeCollection.ElementAt(tupleIndex1);
                            
                                            IDictionary<string, SBRElement> sbrTransactionType_CurrencyCodeTupleElementMap;
                                            sbrTransactionType_CurrencyCodeTupleElementMap = sbrTransactionType_CurrencyCode.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                            
                                            IDS2021.TransactionType.TransactionType_CurrencyCode currencyCode = new IDS2021.TransactionType.TransactionType_CurrencyCode();
                                            transactionType.TransactionType_CurrencyCodeCollection.Add(currencyCode);
                                            currencyCode.OccurrenceIndex = tupleIndex1 + 1;
                                    
                                            if (sbrTransactionType_CurrencyCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.FunctionalCurrency.Code", out currentValue))
                                            {
                                                if (!currentValue.IsNil) currencyCode.IDS423 = currentValue.Value;
                                            }
                                    
                                            if (sbrTransactionType_CurrencyCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeLossesHighestNominal.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) currencyCode.IDS336 = decimal.Parse(currentValue.Value);
                                            }
                                    
                                            if (sbrTransactionType_CurrencyCodeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeGainsHighestNominal.Amount", out currentValue))
                                            {
                                                if (!currentValue.IsNil) currencyCode.IDS337 = decimal.Parse(currentValue.Value);
                                            }
                                } 
                                #endregion End of Tuple sbrTransactionType_CurrencyCodeCollection
                                    } 
                                } 
                        
                                if (sbrTransactionTypeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeLossesOther.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) transactionType.IDS338 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrTransactionTypeTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ForeignExchangeGainsOther.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) transactionType.IDS339 = decimal.Parse(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrTransactionTypeCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.OtherRevenue.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS123 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.CapitalNature.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS129 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ResearchAndDevelopmentCostPlusRemuneration.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS426 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ReceivedOrProvidedServicesPropertyProcessesRightsOrObligationsForNonMonetaryConsideration.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS138 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ShareBasedEmployeeRemuneration.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS147 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.CostContributionArrangements.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS150 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.RestructuringEvents.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS151 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TransactionsRestructuringEventsAssetRevaluation.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS159 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.BranchOperations.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS160 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.FinancialArrangements.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS172 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxationOfFinancialArrangements.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS429 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxationOfFinancialArrangementsRulesApplied.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS177 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxTimingMethodElections.Code", out currentValues))
                    {
                        report.IDS178Collection = new List<string>();
                        try
                        {
                                foreach (SBRElement occurrence in currentValues)
                                {
                                    if (!occurrence.IsNil)
                                        report.IDS178Collection.Add(occurrence.Value);
                                }
                        }
                        catch (Exception)
                        {
                            // The above code is known to throw exceptions in some cases but not others.
                            // We don't really know why but think that catching them like this will result in correct behaviour
                        }
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.InterestInControlledForeignCompaniesAndControlledForeignTrusts.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS184 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestAcquired.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS367 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestDisposed.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS371 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesExcludedTaintedInterestIncome.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS433 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesBankingLicences.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS434 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesIncomeMoneyLending.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS435 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesExcludedTaintedInterestIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS439 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAttributableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS191 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesChangeOfResidenceIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS192 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAndControlledForeignTrustsInterposedAustralianEntitiesAttributableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS193 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestSatisfiedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS446 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestNotSatisfiedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS450 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAssessableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS388 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAllowableDeductionsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS454 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignBranchOperationsInterestsInForeignCompaniesOrForeignTrusts.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS195 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.NonDeductibleExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS199 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncome.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS206 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}IncomeTax.Deduction.DebtEarningNonAssessableNonExemptForeignIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS207 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}CapitalGainsTax.Event.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS208 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}CapitalGainsTax.ActiveForeignBusinessCapitalGains.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS209 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}IncomeTax.ActiveForeignBusinessCapitalGainsReduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS210 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}CapitalGainsTax.ActiveForeignBusinessCapitalLosses.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS211 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}IncomeTax.ActiveForeignBusinessCapitalLossesReduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS212 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.TransferPropertyMoneyOrServices.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS213 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NonResidentTrustInterestEntitlementOrBeneficiary.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS216 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.OffshoreDealingsCompliance.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS393 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ForeignHybridLimitedPartnershipOrForeignHybridCompanyShareholder.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS217 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ForeignHybridLimitedPartnershipOrForeignHybridCompanyInterestIn.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS218 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.Investment.ForeignHybridLimitedPartnershipOrForeignHybridCompanyNetIncomeProfitTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS219 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS220 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.2MillionThresholdTest.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS400 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.90PercentAssetThresholdTest.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS401 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.ExemptionSpecialPurposeEntities.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS402 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.AustralianResidentCompanyElected.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS221 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Identifiers.AustralianBusinessNumber.Identifier", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS222 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.EntityType.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS223 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.GeneralToFinancialStatusChange.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS224 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.AveragingMethodAdopted.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS225 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.DeductionDebt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS226 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.RelatedNonResidentsDebtDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS227 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.DeductionDebtDisallowed.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS228 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.DebtAdjustedAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS229 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.AuthorisedDepositTakingInstitution.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS230 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.EquityCapitalAdjustedAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS231 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.SafeHarbour.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS232 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.EquityShortfall.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS233 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.RiskWeightedAssetsAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS234 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.EquityCapitalAttributableToOverseasPermanentEstablishments.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS235 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.RiskWeightedAssetsOverseasPermanentEstablishmentsAverageValueTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS236 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.ControlledForeignADIEntityEquityAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS237 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.Tier1PrudentialCapitalDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS238 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.RiskWeightedAssetsAttributableToAustralianPermanentEstablishmentsAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS239 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.DotationCapitalAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS240 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.NonAuthorisedDepositTakingInstitution.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS241 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.AssetsValueAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS242 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.AssetRevaluation.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS243 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.DebtMaximumAllowable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS244 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.ExcessDebt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS245 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.NonDebtLiabilitiesAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS246 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.AssociateEntityDebtAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS247 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.AssociateEntityEquityAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS248 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.AssociateEntityExcessAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS249 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.ExcludedEquityInterestsAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS250 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.ZeroCapitalAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS251 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.OnLentAverage.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS252 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.ControlledForeignNonAuthorisedDepositTakingInstitutionEntityEquityAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS253 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.ControlledForeignEntityDebtAverageValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS254 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.ArmsLengthTests.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS255 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.ArmsLengthDebt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS256 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.GearingDebtCapitalTest.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS257 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.WorldwideGroupCapitalRatio.Fraction", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS258 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.WorldwideCapital.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS259 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.WorldwideDebtTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS260 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.ThinCapitalisation.WorldwideEquity.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS261 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.WorldwideGearingDebt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS262 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.StatementWorldwideAsset.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS323 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Capital.ThinCapitalisation.AverageAustralianAsset.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS324 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ForeignBankOrQualifyingFinancialEntity.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS263 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NonOffshoreBankingUnitNotionalInterest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS265 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingUnitNotionalInterest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS266 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalInterestTaxWithholding.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS267 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalDerivativePaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS408 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalDerivativeReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS409 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalForeignExchangeTransactionPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS410 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalForeignExchangeTransactionReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS411 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ForeignBankOrQualifyingFinancialEntityElectedOut.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS412 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ElectedOutNonOffshoreBankingUnitNotionalInterest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS415 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ElectedOutOffshoreBankingUnitNotionalInterest.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS416 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ElectedOutNotionalInterestTaxWithholding.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS417 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalPaymentDenied.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS455 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalPaymentDenied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS456 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingUnitOrHeadConsolidatedGroup.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS268 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ConduitForeignIncome.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS309 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ConduitForeignIncomeBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS310 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.ConduitForeignIncomeDistributed.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS311 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.UnfrankedNonPortfolioDividendAccount.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS312 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.UnfrankedNonPortfolioDividendAccount.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS313 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.UnfrankedNonPortfolioAccountConduitForeignIncomeIncluded.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS314 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.DoubleTaxAgreementFinancialInstitutionsWithholdingExemptionCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS317 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatch.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS481 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDeductionNonInclusion.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS458 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchPayments.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS482 = decimal.Parse(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrMaterialArrangementsCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}MaterialArrangements", out currentValue))
                        sbrMaterialArrangementsCollection = currentValue.Occurrences;
                    else
                        sbrMaterialArrangementsCollection = null;
            
                    #region sbrMaterialArrangementsCollection
            
                    if (sbrMaterialArrangementsCollection != null)
                    {
                        report.MaterialArrangementsCollectionExists = true;
                        report.MaterialArrangementsCollectionCount = sbrMaterialArrangementsCollection.Count();
                        if (sbrMaterialArrangementsCollection != null && sbrMaterialArrangementsCollection.Count() > 0)
                        {
                            report.MaterialArrangementsCollection = new List<IDS2021.MaterialArrangements>();
                            for (int tupleIndex = 0; tupleIndex < sbrMaterialArrangementsCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrMaterialArrangements;
                                sbrMaterialArrangements = sbrMaterialArrangementsCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrMaterialArrangementsTupleElementMap;
                                sbrMaterialArrangementsTupleElementMap = sbrMaterialArrangements.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.MaterialArrangements materialArrangements = new IDS2021.MaterialArrangements();
                                report.MaterialArrangementsCollection.Add(materialArrangements);
                                materialArrangements.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchArrangement.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS460 = currentValue.Value;
                                }
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS461 = currentValue.Value;
                                }
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchPayments.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS462 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDeductionsDeniedOrIncomeInclusion.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS463 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDifferenceReason.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS464 = currentValue.Value;
                                }
                        
                                if (sbrMaterialArrangementsTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchPaymentRecipient.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) materialArrangements.IDS465 = ToBoolean(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrMaterialArrangementsCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.OffshoreHybridMismatch.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS466 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDeductionsDenied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS467 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PaymentNotTaxedOrTaxed10PercentOrLess.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS468 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrHybridPayerMismatchCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}HybridPayerMismatch", out currentValue))
                        sbrHybridPayerMismatchCollection = currentValue.Occurrences;
                    else
                        sbrHybridPayerMismatchCollection = null;
            
                    #region sbrHybridPayerMismatchCollection
            
                    if (sbrHybridPayerMismatchCollection != null)
                    {
                        report.HybridPayerMismatchCollectionExists = true;
                        report.HybridPayerMismatchCollectionCount = sbrHybridPayerMismatchCollection.Count();
                        if (sbrHybridPayerMismatchCollection != null && sbrHybridPayerMismatchCollection.Count() > 0)
                        {
                            report.HybridPayerMismatchCollection = new List<IDS2021.HybridPayerMismatch>();
                            for (int tupleIndex = 0; tupleIndex < sbrHybridPayerMismatchCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrHybridPayerMismatch;
                                sbrHybridPayerMismatch = sbrHybridPayerMismatchCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrHybridPayerMismatchTupleElementMap;
                                sbrHybridPayerMismatchTupleElementMap = sbrHybridPayerMismatch.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.HybridPayerMismatch hybridPayerMismatch = new IDS2021.HybridPayerMismatch();
                                report.HybridPayerMismatchCollection.Add(hybridPayerMismatch);
                                hybridPayerMismatch.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrHybridPayerMismatchTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridPayerMismatch.IDS470 = currentValue.Value;
                                }
                        
                                if (sbrHybridPayerMismatchTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchPayments.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridPayerMismatch.IDS471 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrHybridPayerMismatchTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDeductionsDenied.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridPayerMismatch.IDS472 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrHybridPayerMismatchTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchExceptionReason.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridPayerMismatch.IDS473 = currentValue.Value;
                                }
                    } 
                    #endregion End of Tuple sbrHybridPayerMismatchCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridArrangementRestructureReplacement.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS474 = ToBoolean(currentValues[0].Value);
                    }
        
                    IEnumerable<SBRElement> sbrHybridArrangementRestructureCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}HybridArrangementRestructure", out currentValue))
                        sbrHybridArrangementRestructureCollection = currentValue.Occurrences;
                    else
                        sbrHybridArrangementRestructureCollection = null;
            
                    #region sbrHybridArrangementRestructureCollection
            
                    if (sbrHybridArrangementRestructureCollection != null)
                    {
                        report.HybridArrangementRestructureCollectionExists = true;
                        report.HybridArrangementRestructureCollectionCount = sbrHybridArrangementRestructureCollection.Count();
                        if (sbrHybridArrangementRestructureCollection != null && sbrHybridArrangementRestructureCollection.Count() > 0)
                        {
                            report.HybridArrangementRestructureCollection = new List<IDS2021.HybridArrangementRestructure>();
                            for (int tupleIndex = 0; tupleIndex < sbrHybridArrangementRestructureCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrHybridArrangementRestructure;
                                sbrHybridArrangementRestructure = sbrHybridArrangementRestructureCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrHybridArrangementRestructureTupleElementMap;
                                sbrHybridArrangementRestructureTupleElementMap = sbrHybridArrangementRestructure.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.HybridArrangementRestructure hybridArrangementRestructure = new IDS2021.HybridArrangementRestructure();
                                report.HybridArrangementRestructureCollection.Add(hybridArrangementRestructure);
                                hybridArrangementRestructure.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrHybridArrangementRestructureTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.Description.Text", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridArrangementRestructure.IDS475 = currentValue.Value;
                                }
                        
                                if (sbrHybridArrangementRestructureTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.HybridEventAnalysis.Indicator", out currentValue))
                                {
                                    if (!currentValue.IsNil) hybridArrangementRestructure.IDS476 = ToBoolean(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrHybridArrangementRestructureCollection
                        } 
                    } 
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignEquityDistributionForeignTaxDeduction.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS477 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignDeductionNotNonAssessableNonExempt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS478 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.DerivedBranchHybridMismatchIncome.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS479 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignHybridIncomeNotNonAssessableNonExempt.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS480 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPContext
    
            #region sbrRPQuarterlyContext
            IEnumerable<SBRContext> sbrRPQuarterlyContexts = null;
            SBRContext sbrRPQuarterlyContext = null;
    
            sbrRPQuarterlyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}PeriodTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Quarterly")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPQuarterlyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPQuarterlyContexts != null && sbrRPQuarterlyContexts.Count() > 0)
            {
                sbrRPQuarterlyContext = sbrRPQuarterlyContexts.First();
        
                report.RPQuarterlyId = sbrRPQuarterlyContext.Id;
                report.RPQuarterlyIdentifier = sbrRPQuarterlyContext.EntityIdentifier;
                report.RPQuarterlyIdentifierScheme = sbrRPQuarterlyContext.EntityScheme;
        
                report.RPQuarterlyCount = sbrRPQuarterlyContexts.Count();
                report.RPQuarterlyExists = true;
                report.RPQuarterlyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPQuarterlyContext) + 1;
                report.RPQuarterlyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPQuarterlyContexts.Last()) + 1;
        

                if (report.RPQuarterlyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPQuarterlyIdentifierTFN = report.RPQuarterlyIdentifier;
    
                report.RPQuarterlyStartDate = ((SBRDurationPeriod)sbrRPQuarterlyContext.Period).DurationStartAsDateObject;
                report.RPQuarterlyEndDate = ((SBRDurationPeriod)sbrRPQuarterlyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPQuarterlyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPQuarterlyContexts.Count(); i++)
                {
                    sbrRPQuarterlyContext = sbrRPQuarterlyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPQuarterlyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPQuarterlyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPQuarterlyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPQuarterlyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Interest.DebtReceivedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS173 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.Interest.DebtProvidedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS174 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Interest.EquityReceivedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS175 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.Interest.EquityProvidedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS176 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalAverageBorrowings.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS264 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.NotionalAverageBorrowings.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS407 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ElectedOutNotionalAverageBorrowings.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS413 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPQuarterlyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.ElectedOutNotionalAverageBorrowings.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS414 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPQuarterlyContext
    
            #region sbrRPAssetManagementContext
            IEnumerable<SBRContext> sbrRPAssetManagementContexts = null;
            SBRContext sbrRPAssetManagementContext = null;
    
            sbrRPAssetManagementContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}AssetManagement")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPAssetManagementElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPAssetManagementContexts != null && sbrRPAssetManagementContexts.Count() > 0)
            {
                sbrRPAssetManagementContext = sbrRPAssetManagementContexts.First();
        
                report.RPAssetManagementId = sbrRPAssetManagementContext.Id;
                report.RPAssetManagementIdentifier = sbrRPAssetManagementContext.EntityIdentifier;
                report.RPAssetManagementIdentifierScheme = sbrRPAssetManagementContext.EntityScheme;
        
                report.RPAssetManagementCount = sbrRPAssetManagementContexts.Count();
                report.RPAssetManagementExists = true;
                report.RPAssetManagementOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAssetManagementContext) + 1;
                report.RPAssetManagementLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAssetManagementContexts.Last()) + 1;
        

                if (report.RPAssetManagementIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPAssetManagementIdentifierTFN = report.RPAssetManagementIdentifier;
    
                report.RPAssetManagementStartDate = ((SBRDurationPeriod)sbrRPAssetManagementContext.Period).DurationStartAsDateObject;
                report.RPAssetManagementEndDate = ((SBRDurationPeriod)sbrRPAssetManagementContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPAssetManagementElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPAssetManagementContexts.Count(); i++)
                {
                    sbrRPAssetManagementContext = sbrRPAssetManagementContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPAssetManagementContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPAssetManagementElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPAssetManagementElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPAssetManagementElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPAssetManagementElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS72 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPAssetManagementElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS73 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPAssetManagementElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS74 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPAssetManagementElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS75 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPAssetManagementContext
    
            #region sbrRPBranchOperationsContext
            IEnumerable<SBRContext> sbrRPBranchOperationsContexts = null;
            SBRContext sbrRPBranchOperationsContext = null;
    
            sbrRPBranchOperationsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}BranchOperations")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBranchOperationsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBranchOperationsContexts != null && sbrRPBranchOperationsContexts.Count() > 0)
            {
                sbrRPBranchOperationsContext = sbrRPBranchOperationsContexts.First();
        
                report.RPBranchOperationsId = sbrRPBranchOperationsContext.Id;
                report.RPBranchOperationsIdentifier = sbrRPBranchOperationsContext.EntityIdentifier;
                report.RPBranchOperationsIdentifierScheme = sbrRPBranchOperationsContext.EntityScheme;
        
                report.RPBranchOperationsCount = sbrRPBranchOperationsContexts.Count();
                report.RPBranchOperationsExists = true;
                report.RPBranchOperationsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsContext) + 1;
                report.RPBranchOperationsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsContexts.Last()) + 1;
        

                if (report.RPBranchOperationsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPBranchOperationsIdentifierTFN = report.RPBranchOperationsIdentifier;
    
                report.RPBranchOperationsStartDate = ((SBRDurationPeriod)sbrRPBranchOperationsContext.Period).DurationStartAsDateObject;
                report.RPBranchOperationsEndDate = ((SBRDurationPeriod)sbrRPBranchOperationsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPBranchOperationsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPBranchOperationsContexts.Count(); i++)
                {
                    sbrRPBranchOperationsContext = sbrRPBranchOperationsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBranchOperationsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBranchOperationsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBranchOperationsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBranchOperationsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPBranchOperationsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.TradingStockPurchaseCosts.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS167 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.TradingStockSalesTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS168 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPBranchOperationsContext
    
            #region sbrRPCapitalNonMonetaryPaymentContext
            IEnumerable<SBRContext> sbrRPCapitalNonMonetaryPaymentContexts = null;
            SBRContext sbrRPCapitalNonMonetaryPaymentContext = null;
    
            sbrRPCapitalNonMonetaryPaymentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}CapitalNonMonetaryPayment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCapitalNonMonetaryPaymentContexts != null && sbrRPCapitalNonMonetaryPaymentContexts.Count() > 0)
            {
                sbrRPCapitalNonMonetaryPaymentContext = sbrRPCapitalNonMonetaryPaymentContexts.First();
        
                report.RPCapitalNonMonetaryPaymentId = sbrRPCapitalNonMonetaryPaymentContext.Id;
                report.RPCapitalNonMonetaryPaymentIdentifier = sbrRPCapitalNonMonetaryPaymentContext.EntityIdentifier;
                report.RPCapitalNonMonetaryPaymentIdentifierScheme = sbrRPCapitalNonMonetaryPaymentContext.EntityScheme;
        
                report.RPCapitalNonMonetaryPaymentCount = sbrRPCapitalNonMonetaryPaymentContexts.Count();
                report.RPCapitalNonMonetaryPaymentExists = true;
                report.RPCapitalNonMonetaryPaymentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCapitalNonMonetaryPaymentContext) + 1;
                report.RPCapitalNonMonetaryPaymentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCapitalNonMonetaryPaymentContexts.Last()) + 1;
        

                if (report.RPCapitalNonMonetaryPaymentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCapitalNonMonetaryPaymentIdentifierTFN = report.RPCapitalNonMonetaryPaymentIdentifier;
    
                report.RPCapitalNonMonetaryPaymentStartDate = ((SBRDurationPeriod)sbrRPCapitalNonMonetaryPaymentContext.Period).DurationStartAsDateObject;
                report.RPCapitalNonMonetaryPaymentEndDate = ((SBRDurationPeriod)sbrRPCapitalNonMonetaryPaymentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCapitalNonMonetaryPaymentContexts.Count(); i++)
                {
                    sbrRPCapitalNonMonetaryPaymentContext = sbrRPCapitalNonMonetaryPaymentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCapitalNonMonetaryPaymentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS143 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPCapitalNonMonetaryPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NatureOfItem.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS144 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPCapitalNonMonetaryPaymentContext
    
            #region sbrRPCapitalNoPaymentContext
            IEnumerable<SBRContext> sbrRPCapitalNoPaymentContexts = null;
            SBRContext sbrRPCapitalNoPaymentContext = null;
    
            sbrRPCapitalNoPaymentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}CapitalNoPayment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCapitalNoPaymentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCapitalNoPaymentContexts != null && sbrRPCapitalNoPaymentContexts.Count() > 0)
            {
                sbrRPCapitalNoPaymentContext = sbrRPCapitalNoPaymentContexts.First();
        
                report.RPCapitalNoPaymentId = sbrRPCapitalNoPaymentContext.Id;
                report.RPCapitalNoPaymentIdentifier = sbrRPCapitalNoPaymentContext.EntityIdentifier;
                report.RPCapitalNoPaymentIdentifierScheme = sbrRPCapitalNoPaymentContext.EntityScheme;
        
                report.RPCapitalNoPaymentCount = sbrRPCapitalNoPaymentContexts.Count();
                report.RPCapitalNoPaymentExists = true;
                report.RPCapitalNoPaymentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCapitalNoPaymentContext) + 1;
                report.RPCapitalNoPaymentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCapitalNoPaymentContexts.Last()) + 1;
        

                if (report.RPCapitalNoPaymentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCapitalNoPaymentIdentifierTFN = report.RPCapitalNoPaymentIdentifier;
    
                report.RPCapitalNoPaymentStartDate = ((SBRDurationPeriod)sbrRPCapitalNoPaymentContext.Period).DurationStartAsDateObject;
                report.RPCapitalNoPaymentEndDate = ((SBRDurationPeriod)sbrRPCapitalNoPaymentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCapitalNoPaymentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCapitalNoPaymentContexts.Count(); i++)
                {
                    sbrRPCapitalNoPaymentContext = sbrRPCapitalNoPaymentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCapitalNoPaymentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCapitalNoPaymentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCapitalNoPaymentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCapitalNoPaymentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCapitalNoPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS139 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPCapitalNoPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NatureOfItem.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS140 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPCapitalNoPaymentContext
    
            #region sbrRPDebtFactoringContext
            IEnumerable<SBRContext> sbrRPDebtFactoringContexts = null;
            SBRContext sbrRPDebtFactoringContext = null;
    
            sbrRPDebtFactoringContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}DebtFactoring")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPDebtFactoringElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPDebtFactoringContexts != null && sbrRPDebtFactoringContexts.Count() > 0)
            {
                sbrRPDebtFactoringContext = sbrRPDebtFactoringContexts.First();
        
                report.RPDebtFactoringId = sbrRPDebtFactoringContext.Id;
                report.RPDebtFactoringIdentifier = sbrRPDebtFactoringContext.EntityIdentifier;
                report.RPDebtFactoringIdentifierScheme = sbrRPDebtFactoringContext.EntityScheme;
        
                report.RPDebtFactoringCount = sbrRPDebtFactoringContexts.Count();
                report.RPDebtFactoringExists = true;
                report.RPDebtFactoringOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDebtFactoringContext) + 1;
                report.RPDebtFactoringLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDebtFactoringContexts.Last()) + 1;
        

                if (report.RPDebtFactoringIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPDebtFactoringIdentifierTFN = report.RPDebtFactoringIdentifier;
    
                report.RPDebtFactoringStartDate = ((SBRDurationPeriod)sbrRPDebtFactoringContext.Period).DurationStartAsDateObject;
                report.RPDebtFactoringEndDate = ((SBRDurationPeriod)sbrRPDebtFactoringContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPDebtFactoringElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPDebtFactoringContexts.Count(); i++)
                {
                    sbrRPDebtFactoringContext = sbrRPDebtFactoringContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPDebtFactoringContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPDebtFactoringElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPDebtFactoringElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPDebtFactoringElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPDebtFactoringElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.BookValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS89 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPDebtFactoringElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Consideration.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS90 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPDebtFactoringElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS91 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPDebtFactoringElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS92 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPDebtFactoringContext
    
            #region sbrRPDerivativesContext
            IEnumerable<SBRContext> sbrRPDerivativesContexts = null;
            SBRContext sbrRPDerivativesContext = null;
    
            sbrRPDerivativesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Derivatives")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPDerivativesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPDerivativesContexts != null && sbrRPDerivativesContexts.Count() > 0)
            {
                sbrRPDerivativesContext = sbrRPDerivativesContexts.First();
        
                report.RPDerivativesId = sbrRPDerivativesContext.Id;
                report.RPDerivativesIdentifier = sbrRPDerivativesContext.EntityIdentifier;
                report.RPDerivativesIdentifierScheme = sbrRPDerivativesContext.EntityScheme;
        
                report.RPDerivativesCount = sbrRPDerivativesContexts.Count();
                report.RPDerivativesExists = true;
                report.RPDerivativesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDerivativesContext) + 1;
                report.RPDerivativesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDerivativesContexts.Last()) + 1;
        

                if (report.RPDerivativesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPDerivativesIdentifierTFN = report.RPDerivativesIdentifier;
    
                report.RPDerivativesStartDate = ((SBRDurationPeriod)sbrRPDerivativesContext.Period).DurationStartAsDateObject;
                report.RPDerivativesEndDate = ((SBRDurationPeriod)sbrRPDerivativesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPDerivativesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPDerivativesContexts.Count(); i++)
                {
                    sbrRPDerivativesContext = sbrRPDerivativesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPDerivativesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPDerivativesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPDerivativesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPDerivativesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPDerivativesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS82 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPDerivativesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS83 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPDerivativesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS84 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPDerivativesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS85 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPDerivativesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.RelatedPartyTransactionType.Code", out currentValues))
                    {
                        report.IDS332Collection = new List<string>();
                        try
                        {
                                foreach (SBRElement occurrence in currentValues)
                                {
                                    if (!occurrence.IsNil)
                                        report.IDS332Collection.Add(occurrence.Value);
                                }
                        }
                        catch (Exception)
                        {
                            // The above code is known to throw exceptions in some cases but not others.
                            // We don't really know why but think that catching them like this will result in correct behaviour
                        }
                    }
                } 
            } 
            #endregion End of Context sbrRPDerivativesContext
    
            #region sbrRPInsuranceContext
            IEnumerable<SBRContext> sbrRPInsuranceContexts = null;
            SBRContext sbrRPInsuranceContext = null;
    
            sbrRPInsuranceContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Insurance")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInsuranceElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInsuranceContexts != null && sbrRPInsuranceContexts.Count() > 0)
            {
                sbrRPInsuranceContext = sbrRPInsuranceContexts.First();
        
                report.RPInsuranceId = sbrRPInsuranceContext.Id;
                report.RPInsuranceIdentifier = sbrRPInsuranceContext.EntityIdentifier;
                report.RPInsuranceIdentifierScheme = sbrRPInsuranceContext.EntityScheme;
        
                report.RPInsuranceCount = sbrRPInsuranceContexts.Count();
                report.RPInsuranceExists = true;
                report.RPInsuranceOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInsuranceContext) + 1;
                report.RPInsuranceLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInsuranceContexts.Last()) + 1;
        

                if (report.RPInsuranceIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInsuranceIdentifierTFN = report.RPInsuranceIdentifier;
    
                report.RPInsuranceStartDate = ((SBRDurationPeriod)sbrRPInsuranceContext.Period).DurationStartAsDateObject;
                report.RPInsuranceEndDate = ((SBRDurationPeriod)sbrRPInsuranceContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPInsuranceElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInsuranceContexts.Count(); i++)
                {
                    sbrRPInsuranceContext = sbrRPInsuranceContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInsuranceContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInsuranceElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInsuranceElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInsuranceElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS44 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS45 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS46 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS47 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPInsuranceContext
    
            #region sbrRPLicenceFeesContext
            IEnumerable<SBRContext> sbrRPLicenceFeesContexts = null;
            SBRContext sbrRPLicenceFeesContext = null;
    
            sbrRPLicenceFeesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}LicenceFees")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLicenceFeesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLicenceFeesContexts != null && sbrRPLicenceFeesContexts.Count() > 0)
            {
                sbrRPLicenceFeesContext = sbrRPLicenceFeesContexts.First();
        
                report.RPLicenceFeesId = sbrRPLicenceFeesContext.Id;
                report.RPLicenceFeesIdentifier = sbrRPLicenceFeesContext.EntityIdentifier;
                report.RPLicenceFeesIdentifierScheme = sbrRPLicenceFeesContext.EntityScheme;
        
                report.RPLicenceFeesCount = sbrRPLicenceFeesContexts.Count();
                report.RPLicenceFeesExists = true;
                report.RPLicenceFeesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLicenceFeesContext) + 1;
                report.RPLicenceFeesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLicenceFeesContexts.Last()) + 1;
        

                if (report.RPLicenceFeesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLicenceFeesIdentifierTFN = report.RPLicenceFeesIdentifier;
    
                report.RPLicenceFeesStartDate = ((SBRDurationPeriod)sbrRPLicenceFeesContext.Period).DurationStartAsDateObject;
                report.RPLicenceFeesEndDate = ((SBRDurationPeriod)sbrRPLicenceFeesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLicenceFeesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLicenceFeesContexts.Count(); i++)
                {
                    sbrRPLicenceFeesContext = sbrRPLicenceFeesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLicenceFeesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLicenceFeesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLicenceFeesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLicenceFeesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLicenceFeesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.DeductionsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS26 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLicenceFeesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.IncludedInAssessableTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS27 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLicenceFeesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS28 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPLicenceFeesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS29 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPLicenceFeesContext
    
            #region sbrRPLogisticsContext
            IEnumerable<SBRContext> sbrRPLogisticsContexts = null;
            SBRContext sbrRPLogisticsContext = null;
    
            sbrRPLogisticsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Logistics")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLogisticsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLogisticsContexts != null && sbrRPLogisticsContexts.Count() > 0)
            {
                sbrRPLogisticsContext = sbrRPLogisticsContexts.First();
        
                report.RPLogisticsId = sbrRPLogisticsContext.Id;
                report.RPLogisticsIdentifier = sbrRPLogisticsContext.EntityIdentifier;
                report.RPLogisticsIdentifierScheme = sbrRPLogisticsContext.EntityScheme;
        
                report.RPLogisticsCount = sbrRPLogisticsContexts.Count();
                report.RPLogisticsExists = true;
                report.RPLogisticsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLogisticsContext) + 1;
                report.RPLogisticsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLogisticsContexts.Last()) + 1;
        

                if (report.RPLogisticsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLogisticsIdentifierTFN = report.RPLogisticsIdentifier;
    
                report.RPLogisticsStartDate = ((SBRDurationPeriod)sbrRPLogisticsContext.Period).DurationStartAsDateObject;
                report.RPLogisticsEndDate = ((SBRDurationPeriod)sbrRPLogisticsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLogisticsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLogisticsContexts.Count(); i++)
                {
                    sbrRPLogisticsContext = sbrRPLogisticsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLogisticsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLogisticsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLogisticsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLogisticsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLogisticsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS68 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLogisticsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS69 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLogisticsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS70 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPLogisticsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS71 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPLogisticsContext
    
            #region sbrRPManagementAndAdminContext
            IEnumerable<SBRContext> sbrRPManagementAndAdminContexts = null;
            SBRContext sbrRPManagementAndAdminContext = null;
    
            sbrRPManagementAndAdminContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ManagementAndAdmin")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPManagementAndAdminElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPManagementAndAdminContexts != null && sbrRPManagementAndAdminContexts.Count() > 0)
            {
                sbrRPManagementAndAdminContext = sbrRPManagementAndAdminContexts.First();
        
                report.RPManagementAndAdminId = sbrRPManagementAndAdminContext.Id;
                report.RPManagementAndAdminIdentifier = sbrRPManagementAndAdminContext.EntityIdentifier;
                report.RPManagementAndAdminIdentifierScheme = sbrRPManagementAndAdminContext.EntityScheme;
        
                report.RPManagementAndAdminCount = sbrRPManagementAndAdminContexts.Count();
                report.RPManagementAndAdminExists = true;
                report.RPManagementAndAdminOccurrenceIndex = Array.IndexOf(contextArray, sbrRPManagementAndAdminContext) + 1;
                report.RPManagementAndAdminLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPManagementAndAdminContexts.Last()) + 1;
        

                if (report.RPManagementAndAdminIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPManagementAndAdminIdentifierTFN = report.RPManagementAndAdminIdentifier;
    
                report.RPManagementAndAdminStartDate = ((SBRDurationPeriod)sbrRPManagementAndAdminContext.Period).DurationStartAsDateObject;
                report.RPManagementAndAdminEndDate = ((SBRDurationPeriod)sbrRPManagementAndAdminContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPManagementAndAdminElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPManagementAndAdminContexts.Count(); i++)
                {
                    sbrRPManagementAndAdminContext = sbrRPManagementAndAdminContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPManagementAndAdminContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPManagementAndAdminElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPManagementAndAdminElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPManagementAndAdminElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPManagementAndAdminElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS40 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPManagementAndAdminElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS41 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPManagementAndAdminElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS42 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPManagementAndAdminElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS43 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPManagementAndAdminContext
    
            #region sbrRPOtherFinancialDealingsContext
            IEnumerable<SBRContext> sbrRPOtherFinancialDealingsContexts = null;
            SBRContext sbrRPOtherFinancialDealingsContext = null;
    
            sbrRPOtherFinancialDealingsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherFinancialDealings")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherFinancialDealingsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherFinancialDealingsContexts != null && sbrRPOtherFinancialDealingsContexts.Count() > 0)
            {
                sbrRPOtherFinancialDealingsContext = sbrRPOtherFinancialDealingsContexts.First();
        
                report.RPOtherFinancialDealingsId = sbrRPOtherFinancialDealingsContext.Id;
                report.RPOtherFinancialDealingsIdentifier = sbrRPOtherFinancialDealingsContext.EntityIdentifier;
                report.RPOtherFinancialDealingsIdentifierScheme = sbrRPOtherFinancialDealingsContext.EntityScheme;
        
                report.RPOtherFinancialDealingsCount = sbrRPOtherFinancialDealingsContexts.Count();
                report.RPOtherFinancialDealingsExists = true;
                report.RPOtherFinancialDealingsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsContext) + 1;
                report.RPOtherFinancialDealingsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsContexts.Last()) + 1;
        

                if (report.RPOtherFinancialDealingsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherFinancialDealingsIdentifierTFN = report.RPOtherFinancialDealingsIdentifier;
    
                report.RPOtherFinancialDealingsStartDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsContext.Period).DurationStartAsDateObject;
                report.RPOtherFinancialDealingsEndDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherFinancialDealingsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherFinancialDealingsContexts.Count(); i++)
                {
                    sbrRPOtherFinancialDealingsContext = sbrRPOtherFinancialDealingsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherFinancialDealingsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherFinancialDealingsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherFinancialDealingsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherFinancialDealingsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS118 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS119 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS120 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS121 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.Description.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS122 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherFinancialDealingsContext
    
            #region sbrRPOtherGuaranteesContext
            IEnumerable<SBRContext> sbrRPOtherGuaranteesContexts = null;
            SBRContext sbrRPOtherGuaranteesContext = null;
    
            sbrRPOtherGuaranteesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherGuarantees")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherGuaranteesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherGuaranteesContexts != null && sbrRPOtherGuaranteesContexts.Count() > 0)
            {
                sbrRPOtherGuaranteesContext = sbrRPOtherGuaranteesContexts.First();
        
                report.RPOtherGuaranteesId = sbrRPOtherGuaranteesContext.Id;
                report.RPOtherGuaranteesIdentifier = sbrRPOtherGuaranteesContext.EntityIdentifier;
                report.RPOtherGuaranteesIdentifierScheme = sbrRPOtherGuaranteesContext.EntityScheme;
        
                report.RPOtherGuaranteesCount = sbrRPOtherGuaranteesContexts.Count();
                report.RPOtherGuaranteesExists = true;
                report.RPOtherGuaranteesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherGuaranteesContext) + 1;
                report.RPOtherGuaranteesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherGuaranteesContexts.Last()) + 1;
        

                if (report.RPOtherGuaranteesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherGuaranteesIdentifierTFN = report.RPOtherGuaranteesIdentifier;
    
                report.RPOtherGuaranteesStartDate = ((SBRDurationPeriod)sbrRPOtherGuaranteesContext.Period).DurationStartAsDateObject;
                report.RPOtherGuaranteesEndDate = ((SBRDurationPeriod)sbrRPOtherGuaranteesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherGuaranteesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherGuaranteesContexts.Count(); i++)
                {
                    sbrRPOtherGuaranteesContext = sbrRPOtherGuaranteesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherGuaranteesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherGuaranteesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherGuaranteesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherGuaranteesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherGuaranteesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS106 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherGuaranteesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS107 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherGuaranteesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS108 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherGuaranteesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS109 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherGuaranteesContext
    
            #region sbrRPOtherInsuranceContext
            IEnumerable<SBRContext> sbrRPOtherInsuranceContexts = null;
            SBRContext sbrRPOtherInsuranceContext = null;
    
            sbrRPOtherInsuranceContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherInsurance")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherInsuranceElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherInsuranceContexts != null && sbrRPOtherInsuranceContexts.Count() > 0)
            {
                sbrRPOtherInsuranceContext = sbrRPOtherInsuranceContexts.First();
        
                report.RPOtherInsuranceId = sbrRPOtherInsuranceContext.Id;
                report.RPOtherInsuranceIdentifier = sbrRPOtherInsuranceContext.EntityIdentifier;
                report.RPOtherInsuranceIdentifierScheme = sbrRPOtherInsuranceContext.EntityScheme;
        
                report.RPOtherInsuranceCount = sbrRPOtherInsuranceContexts.Count();
                report.RPOtherInsuranceExists = true;
                report.RPOtherInsuranceOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherInsuranceContext) + 1;
                report.RPOtherInsuranceLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherInsuranceContexts.Last()) + 1;
        

                if (report.RPOtherInsuranceIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherInsuranceIdentifierTFN = report.RPOtherInsuranceIdentifier;
    
                report.RPOtherInsuranceStartDate = ((SBRDurationPeriod)sbrRPOtherInsuranceContext.Period).DurationStartAsDateObject;
                report.RPOtherInsuranceEndDate = ((SBRDurationPeriod)sbrRPOtherInsuranceContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherInsuranceElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherInsuranceContexts.Count(); i++)
                {
                    sbrRPOtherInsuranceContext = sbrRPOtherInsuranceContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherInsuranceContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherInsuranceElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherInsuranceElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherInsuranceElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS110 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS111 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS112 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherInsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS113 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherInsuranceContext
    
            #region sbrRPOtherInterestContext
            IEnumerable<SBRContext> sbrRPOtherInterestContexts = null;
            SBRContext sbrRPOtherInterestContext = null;
    
            sbrRPOtherInterestContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherInterest")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherInterestElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherInterestContexts != null && sbrRPOtherInterestContexts.Count() > 0)
            {
                sbrRPOtherInterestContext = sbrRPOtherInterestContexts.First();
        
                report.RPOtherInterestId = sbrRPOtherInterestContext.Id;
                report.RPOtherInterestIdentifier = sbrRPOtherInterestContext.EntityIdentifier;
                report.RPOtherInterestIdentifierScheme = sbrRPOtherInterestContext.EntityScheme;
        
                report.RPOtherInterestCount = sbrRPOtherInterestContexts.Count();
                report.RPOtherInterestExists = true;
                report.RPOtherInterestOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherInterestContext) + 1;
                report.RPOtherInterestLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherInterestContexts.Last()) + 1;
        

                if (report.RPOtherInterestIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherInterestIdentifierTFN = report.RPOtherInterestIdentifier;
    
                report.RPOtherInterestStartDate = ((SBRDurationPeriod)sbrRPOtherInterestContext.Period).DurationStartAsDateObject;
                report.RPOtherInterestEndDate = ((SBRDurationPeriod)sbrRPOtherInterestContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherInterestElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherInterestContexts.Count(); i++)
                {
                    sbrRPOtherInterestContext = sbrRPOtherInterestContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherInterestContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherInterestElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherInterestElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherInterestElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherInterestElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS102 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherInterestElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS103 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherInterestElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS104 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherInterestElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS105 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherInterestContext
    
            #region sbrRPOtherNotReportedIntangibleNonRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedIntangibleNonRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedIntangibleNonRevenueContext = null;
    
            sbrRPOtherNotReportedIntangibleNonRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedIntangibleNonRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedIntangibleNonRevenueContexts != null && sbrRPOtherNotReportedIntangibleNonRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedIntangibleNonRevenueContext = sbrRPOtherNotReportedIntangibleNonRevenueContexts.First();
        
                report.RPOtherNotReportedIntangibleNonRevenueId = sbrRPOtherNotReportedIntangibleNonRevenueContext.Id;
                report.RPOtherNotReportedIntangibleNonRevenueIdentifier = sbrRPOtherNotReportedIntangibleNonRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedIntangibleNonRevenueIdentifierScheme = sbrRPOtherNotReportedIntangibleNonRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedIntangibleNonRevenueCount = sbrRPOtherNotReportedIntangibleNonRevenueContexts.Count();
                report.RPOtherNotReportedIntangibleNonRevenueExists = true;
                report.RPOtherNotReportedIntangibleNonRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedIntangibleNonRevenueContext) + 1;
                report.RPOtherNotReportedIntangibleNonRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedIntangibleNonRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedIntangibleNonRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedIntangibleNonRevenueIdentifierTFN = report.RPOtherNotReportedIntangibleNonRevenueIdentifier;
    
                report.RPOtherNotReportedIntangibleNonRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedIntangibleNonRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedIntangibleNonRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedIntangibleNonRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedIntangibleNonRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedIntangibleNonRevenueContext = sbrRPOtherNotReportedIntangibleNonRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedIntangibleNonRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS134 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS135 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.AcquisitionsAndDisposalsPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS136 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS137 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedIntangibleNonRevenueContext
    
            #region sbrRPOtherNotReportedRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedRevenueContext = null;
    
            sbrRPOtherNotReportedRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedRevenueContexts != null && sbrRPOtherNotReportedRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedRevenueContext = sbrRPOtherNotReportedRevenueContexts.First();
        
                report.RPOtherNotReportedRevenueId = sbrRPOtherNotReportedRevenueContext.Id;
                report.RPOtherNotReportedRevenueIdentifier = sbrRPOtherNotReportedRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedRevenueIdentifierScheme = sbrRPOtherNotReportedRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedRevenueCount = sbrRPOtherNotReportedRevenueContexts.Count();
                report.RPOtherNotReportedRevenueExists = true;
                report.RPOtherNotReportedRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedRevenueContext) + 1;
                report.RPOtherNotReportedRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedRevenueIdentifierTFN = report.RPOtherNotReportedRevenueIdentifier;
    
                report.RPOtherNotReportedRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedRevenueContext = sbrRPOtherNotReportedRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS124 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS125 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS126 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS127 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.Description.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS128 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedRevenueContext
    
            #region sbrRPOtherNotReportedTangibleNonRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedTangibleNonRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedTangibleNonRevenueContext = null;
    
            sbrRPOtherNotReportedTangibleNonRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedTangibleNonRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedTangibleNonRevenueContexts != null && sbrRPOtherNotReportedTangibleNonRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedTangibleNonRevenueContext = sbrRPOtherNotReportedTangibleNonRevenueContexts.First();
        
                report.RPOtherNotReportedTangibleNonRevenueId = sbrRPOtherNotReportedTangibleNonRevenueContext.Id;
                report.RPOtherNotReportedTangibleNonRevenueIdentifier = sbrRPOtherNotReportedTangibleNonRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedTangibleNonRevenueIdentifierScheme = sbrRPOtherNotReportedTangibleNonRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedTangibleNonRevenueCount = sbrRPOtherNotReportedTangibleNonRevenueContexts.Count();
                report.RPOtherNotReportedTangibleNonRevenueExists = true;
                report.RPOtherNotReportedTangibleNonRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedTangibleNonRevenueContext) + 1;
                report.RPOtherNotReportedTangibleNonRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedTangibleNonRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedTangibleNonRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedTangibleNonRevenueIdentifierTFN = report.RPOtherNotReportedTangibleNonRevenueIdentifier;
    
                report.RPOtherNotReportedTangibleNonRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedTangibleNonRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedTangibleNonRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedTangibleNonRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedTangibleNonRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedTangibleNonRevenueContext = sbrRPOtherNotReportedTangibleNonRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedTangibleNonRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS130 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS131 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.AcquisitionsAndDisposalsPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS132 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedTangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS133 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedTangibleNonRevenueContext
    
            #region sbrRPOtherReinsuranceContext
            IEnumerable<SBRContext> sbrRPOtherReinsuranceContexts = null;
            SBRContext sbrRPOtherReinsuranceContext = null;
    
            sbrRPOtherReinsuranceContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherReinsurance")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherReinsuranceElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherReinsuranceContexts != null && sbrRPOtherReinsuranceContexts.Count() > 0)
            {
                sbrRPOtherReinsuranceContext = sbrRPOtherReinsuranceContexts.First();
        
                report.RPOtherReinsuranceId = sbrRPOtherReinsuranceContext.Id;
                report.RPOtherReinsuranceIdentifier = sbrRPOtherReinsuranceContext.EntityIdentifier;
                report.RPOtherReinsuranceIdentifierScheme = sbrRPOtherReinsuranceContext.EntityScheme;
        
                report.RPOtherReinsuranceCount = sbrRPOtherReinsuranceContexts.Count();
                report.RPOtherReinsuranceExists = true;
                report.RPOtherReinsuranceOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherReinsuranceContext) + 1;
                report.RPOtherReinsuranceLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherReinsuranceContexts.Last()) + 1;
        

                if (report.RPOtherReinsuranceIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherReinsuranceIdentifierTFN = report.RPOtherReinsuranceIdentifier;
    
                report.RPOtherReinsuranceStartDate = ((SBRDurationPeriod)sbrRPOtherReinsuranceContext.Period).DurationStartAsDateObject;
                report.RPOtherReinsuranceEndDate = ((SBRDurationPeriod)sbrRPOtherReinsuranceContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherReinsuranceElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherReinsuranceContexts.Count(); i++)
                {
                    sbrRPOtherReinsuranceContext = sbrRPOtherReinsuranceContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherReinsuranceContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherReinsuranceElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherReinsuranceElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherReinsuranceElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS114 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS115 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS116 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS117 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherReinsuranceContext
    
            #region sbrRPOtherServicesContext
            IEnumerable<SBRContext> sbrRPOtherServicesContexts = null;
            SBRContext sbrRPOtherServicesContext = null;
    
            sbrRPOtherServicesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherServices")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherServicesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherServicesContexts != null && sbrRPOtherServicesContexts.Count() > 0)
            {
                sbrRPOtherServicesContext = sbrRPOtherServicesContexts.First();
        
                report.RPOtherServicesId = sbrRPOtherServicesContext.Id;
                report.RPOtherServicesIdentifier = sbrRPOtherServicesContext.EntityIdentifier;
                report.RPOtherServicesIdentifierScheme = sbrRPOtherServicesContext.EntityScheme;
        
                report.RPOtherServicesCount = sbrRPOtherServicesContexts.Count();
                report.RPOtherServicesExists = true;
                report.RPOtherServicesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherServicesContext) + 1;
                report.RPOtherServicesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherServicesContexts.Last()) + 1;
        

                if (report.RPOtherServicesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherServicesIdentifierTFN = report.RPOtherServicesIdentifier;
    
                report.RPOtherServicesStartDate = ((SBRDurationPeriod)sbrRPOtherServicesContext.Period).DurationStartAsDateObject;
                report.RPOtherServicesEndDate = ((SBRDurationPeriod)sbrRPOtherServicesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherServicesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherServicesContexts.Count(); i++)
                {
                    sbrRPOtherServicesContext = sbrRPOtherServicesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherServicesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherServicesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherServicesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherServicesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS76 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS77 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS78 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS79 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.Description.Text", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS80 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherServicesContext
    
            #region sbrRPRechargeAmountsContext
            IEnumerable<SBRContext> sbrRPRechargeAmountsContexts = null;
            SBRContext sbrRPRechargeAmountsContext = null;
    
            sbrRPRechargeAmountsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RechargeAmounts")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRechargeAmountsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRechargeAmountsContexts != null && sbrRPRechargeAmountsContexts.Count() > 0)
            {
                sbrRPRechargeAmountsContext = sbrRPRechargeAmountsContexts.First();
        
                report.RPRechargeAmountsId = sbrRPRechargeAmountsContext.Id;
                report.RPRechargeAmountsIdentifier = sbrRPRechargeAmountsContext.EntityIdentifier;
                report.RPRechargeAmountsIdentifierScheme = sbrRPRechargeAmountsContext.EntityScheme;
        
                report.RPRechargeAmountsCount = sbrRPRechargeAmountsContexts.Count();
                report.RPRechargeAmountsExists = true;
                report.RPRechargeAmountsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRechargeAmountsContext) + 1;
                report.RPRechargeAmountsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRechargeAmountsContexts.Last()) + 1;
        

                if (report.RPRechargeAmountsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPRechargeAmountsIdentifierTFN = report.RPRechargeAmountsIdentifier;
    
                report.RPRechargeAmountsStartDate = ((SBRDurationPeriod)sbrRPRechargeAmountsContext.Period).DurationStartAsDateObject;
                report.RPRechargeAmountsEndDate = ((SBRDurationPeriod)sbrRPRechargeAmountsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPRechargeAmountsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPRechargeAmountsContexts.Count(); i++)
                {
                    sbrRPRechargeAmountsContext = sbrRPRechargeAmountsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRechargeAmountsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRechargeAmountsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRechargeAmountsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRechargeAmountsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPRechargeAmountsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.DeductionsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS148 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPRechargeAmountsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.IncludedInAssessableTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS149 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPRechargeAmountsContext
    
            #region sbrRPReinsuranceContext
            IEnumerable<SBRContext> sbrRPReinsuranceContexts = null;
            SBRContext sbrRPReinsuranceContext = null;
    
            sbrRPReinsuranceContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Reinsurance")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPReinsuranceElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPReinsuranceContexts != null && sbrRPReinsuranceContexts.Count() > 0)
            {
                sbrRPReinsuranceContext = sbrRPReinsuranceContexts.First();
        
                report.RPReinsuranceId = sbrRPReinsuranceContext.Id;
                report.RPReinsuranceIdentifier = sbrRPReinsuranceContext.EntityIdentifier;
                report.RPReinsuranceIdentifierScheme = sbrRPReinsuranceContext.EntityScheme;
        
                report.RPReinsuranceCount = sbrRPReinsuranceContexts.Count();
                report.RPReinsuranceExists = true;
                report.RPReinsuranceOccurrenceIndex = Array.IndexOf(contextArray, sbrRPReinsuranceContext) + 1;
                report.RPReinsuranceLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPReinsuranceContexts.Last()) + 1;
        

                if (report.RPReinsuranceIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPReinsuranceIdentifierTFN = report.RPReinsuranceIdentifier;
    
                report.RPReinsuranceStartDate = ((SBRDurationPeriod)sbrRPReinsuranceContext.Period).DurationStartAsDateObject;
                report.RPReinsuranceEndDate = ((SBRDurationPeriod)sbrRPReinsuranceContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPReinsuranceElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPReinsuranceContexts.Count(); i++)
                {
                    sbrRPReinsuranceContext = sbrRPReinsuranceContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPReinsuranceContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPReinsuranceElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPReinsuranceElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPReinsuranceElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS48 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS49 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS50 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPReinsuranceElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS51 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPReinsuranceContext
    
            #region sbrRPRentLeaseContext
            IEnumerable<SBRContext> sbrRPRentLeaseContexts = null;
            SBRContext sbrRPRentLeaseContext = null;
    
            sbrRPRentLeaseContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RentLease")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRentLeaseElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRentLeaseContexts != null && sbrRPRentLeaseContexts.Count() > 0)
            {
                sbrRPRentLeaseContext = sbrRPRentLeaseContexts.First();
        
                report.RPRentLeaseId = sbrRPRentLeaseContext.Id;
                report.RPRentLeaseIdentifier = sbrRPRentLeaseContext.EntityIdentifier;
                report.RPRentLeaseIdentifierScheme = sbrRPRentLeaseContext.EntityScheme;
        
                report.RPRentLeaseCount = sbrRPRentLeaseContexts.Count();
                report.RPRentLeaseExists = true;
                report.RPRentLeaseOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRentLeaseContext) + 1;
                report.RPRentLeaseLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRentLeaseContexts.Last()) + 1;
        

                if (report.RPRentLeaseIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPRentLeaseIdentifierTFN = report.RPRentLeaseIdentifier;
    
                report.RPRentLeaseStartDate = ((SBRDurationPeriod)sbrRPRentLeaseContext.Period).DurationStartAsDateObject;
                report.RPRentLeaseEndDate = ((SBRDurationPeriod)sbrRPRentLeaseContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPRentLeaseElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPRentLeaseContexts.Count(); i++)
                {
                    sbrRPRentLeaseContext = sbrRPRentLeaseContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRentLeaseContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRentLeaseElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRentLeaseElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRentLeaseElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPRentLeaseElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS31 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPRentLeaseElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS32 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPRentLeaseElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS33 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRentLeaseElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS34 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPRentLeaseContext
    
            #region sbrRPResearchAndDevelopmentContext
            IEnumerable<SBRContext> sbrRPResearchAndDevelopmentContexts = null;
            SBRContext sbrRPResearchAndDevelopmentContext = null;
    
            sbrRPResearchAndDevelopmentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ResearchAndDevelopment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPResearchAndDevelopmentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPResearchAndDevelopmentContexts != null && sbrRPResearchAndDevelopmentContexts.Count() > 0)
            {
                sbrRPResearchAndDevelopmentContext = sbrRPResearchAndDevelopmentContexts.First();
        
                report.RPResearchAndDevelopmentId = sbrRPResearchAndDevelopmentContext.Id;
                report.RPResearchAndDevelopmentIdentifier = sbrRPResearchAndDevelopmentContext.EntityIdentifier;
                report.RPResearchAndDevelopmentIdentifierScheme = sbrRPResearchAndDevelopmentContext.EntityScheme;
        
                report.RPResearchAndDevelopmentCount = sbrRPResearchAndDevelopmentContexts.Count();
                report.RPResearchAndDevelopmentExists = true;
                report.RPResearchAndDevelopmentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPResearchAndDevelopmentContext) + 1;
                report.RPResearchAndDevelopmentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPResearchAndDevelopmentContexts.Last()) + 1;
        

                if (report.RPResearchAndDevelopmentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPResearchAndDevelopmentIdentifierTFN = report.RPResearchAndDevelopmentIdentifier;
    
                report.RPResearchAndDevelopmentStartDate = ((SBRDurationPeriod)sbrRPResearchAndDevelopmentContext.Period).DurationStartAsDateObject;
                report.RPResearchAndDevelopmentEndDate = ((SBRDurationPeriod)sbrRPResearchAndDevelopmentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPResearchAndDevelopmentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPResearchAndDevelopmentContexts.Count(); i++)
                {
                    sbrRPResearchAndDevelopmentContext = sbrRPResearchAndDevelopmentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPResearchAndDevelopmentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPResearchAndDevelopmentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPResearchAndDevelopmentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPResearchAndDevelopmentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPResearchAndDevelopmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS52 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPResearchAndDevelopmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS53 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPResearchAndDevelopmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS54 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPResearchAndDevelopmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS55 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPResearchAndDevelopmentContext
    
            #region sbrRPRevenueNonMonetaryPaymentContext
            IEnumerable<SBRContext> sbrRPRevenueNonMonetaryPaymentContexts = null;
            SBRContext sbrRPRevenueNonMonetaryPaymentContext = null;
    
            sbrRPRevenueNonMonetaryPaymentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RevenueNonMonetaryPayment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRevenueNonMonetaryPaymentContexts != null && sbrRPRevenueNonMonetaryPaymentContexts.Count() > 0)
            {
                sbrRPRevenueNonMonetaryPaymentContext = sbrRPRevenueNonMonetaryPaymentContexts.First();
        
                report.RPRevenueNonMonetaryPaymentId = sbrRPRevenueNonMonetaryPaymentContext.Id;
                report.RPRevenueNonMonetaryPaymentIdentifier = sbrRPRevenueNonMonetaryPaymentContext.EntityIdentifier;
                report.RPRevenueNonMonetaryPaymentIdentifierScheme = sbrRPRevenueNonMonetaryPaymentContext.EntityScheme;
        
                report.RPRevenueNonMonetaryPaymentCount = sbrRPRevenueNonMonetaryPaymentContexts.Count();
                report.RPRevenueNonMonetaryPaymentExists = true;
                report.RPRevenueNonMonetaryPaymentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRevenueNonMonetaryPaymentContext) + 1;
                report.RPRevenueNonMonetaryPaymentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRevenueNonMonetaryPaymentContexts.Last()) + 1;
        

                if (report.RPRevenueNonMonetaryPaymentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPRevenueNonMonetaryPaymentIdentifierTFN = report.RPRevenueNonMonetaryPaymentIdentifier;
    
                report.RPRevenueNonMonetaryPaymentStartDate = ((SBRDurationPeriod)sbrRPRevenueNonMonetaryPaymentContext.Period).DurationStartAsDateObject;
                report.RPRevenueNonMonetaryPaymentEndDate = ((SBRDurationPeriod)sbrRPRevenueNonMonetaryPaymentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPRevenueNonMonetaryPaymentContexts.Count(); i++)
                {
                    sbrRPRevenueNonMonetaryPaymentContext = sbrRPRevenueNonMonetaryPaymentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRevenueNonMonetaryPaymentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS145 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRevenueNonMonetaryPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NatureOfItem.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS146 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPRevenueNonMonetaryPaymentContext
    
            #region sbrRPRevenueNoPaymentContext
            IEnumerable<SBRContext> sbrRPRevenueNoPaymentContexts = null;
            SBRContext sbrRPRevenueNoPaymentContext = null;
    
            sbrRPRevenueNoPaymentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RevenueNoPayment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRevenueNoPaymentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRevenueNoPaymentContexts != null && sbrRPRevenueNoPaymentContexts.Count() > 0)
            {
                sbrRPRevenueNoPaymentContext = sbrRPRevenueNoPaymentContexts.First();
        
                report.RPRevenueNoPaymentId = sbrRPRevenueNoPaymentContext.Id;
                report.RPRevenueNoPaymentIdentifier = sbrRPRevenueNoPaymentContext.EntityIdentifier;
                report.RPRevenueNoPaymentIdentifierScheme = sbrRPRevenueNoPaymentContext.EntityScheme;
        
                report.RPRevenueNoPaymentCount = sbrRPRevenueNoPaymentContexts.Count();
                report.RPRevenueNoPaymentExists = true;
                report.RPRevenueNoPaymentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRevenueNoPaymentContext) + 1;
                report.RPRevenueNoPaymentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRevenueNoPaymentContexts.Last()) + 1;
        

                if (report.RPRevenueNoPaymentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPRevenueNoPaymentIdentifierTFN = report.RPRevenueNoPaymentIdentifier;
    
                report.RPRevenueNoPaymentStartDate = ((SBRDurationPeriod)sbrRPRevenueNoPaymentContext.Period).DurationStartAsDateObject;
                report.RPRevenueNoPaymentEndDate = ((SBRDurationPeriod)sbrRPRevenueNoPaymentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPRevenueNoPaymentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPRevenueNoPaymentContexts.Count(); i++)
                {
                    sbrRPRevenueNoPaymentContext = sbrRPRevenueNoPaymentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRevenueNoPaymentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRevenueNoPaymentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRevenueNoPaymentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRevenueNoPaymentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPRevenueNoPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS141 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRevenueNoPaymentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NatureOfItem.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS142 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPRevenueNoPaymentContext
    
            #region sbrRPRoyaltiesContext
            IEnumerable<SBRContext> sbrRPRoyaltiesContexts = null;
            SBRContext sbrRPRoyaltiesContext = null;
    
            sbrRPRoyaltiesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Royalties")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRoyaltiesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRoyaltiesContexts != null && sbrRPRoyaltiesContexts.Count() > 0)
            {
                sbrRPRoyaltiesContext = sbrRPRoyaltiesContexts.First();
        
                report.RPRoyaltiesId = sbrRPRoyaltiesContext.Id;
                report.RPRoyaltiesIdentifier = sbrRPRoyaltiesContext.EntityIdentifier;
                report.RPRoyaltiesIdentifierScheme = sbrRPRoyaltiesContext.EntityScheme;
        
                report.RPRoyaltiesCount = sbrRPRoyaltiesContexts.Count();
                report.RPRoyaltiesExists = true;
                report.RPRoyaltiesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRoyaltiesContext) + 1;
                report.RPRoyaltiesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRoyaltiesContexts.Last()) + 1;
        

                if (report.RPRoyaltiesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPRoyaltiesIdentifierTFN = report.RPRoyaltiesIdentifier;
    
                report.RPRoyaltiesStartDate = ((SBRDurationPeriod)sbrRPRoyaltiesContext.Period).DurationStartAsDateObject;
                report.RPRoyaltiesEndDate = ((SBRDurationPeriod)sbrRPRoyaltiesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPRoyaltiesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPRoyaltiesContexts.Count(); i++)
                {
                    sbrRPRoyaltiesContext = sbrRPRoyaltiesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRoyaltiesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRoyaltiesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRoyaltiesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRoyaltiesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPRoyaltiesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.DeductionsTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS22 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPRoyaltiesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.IncludedInAssessableTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS23 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPRoyaltiesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS24 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRoyaltiesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS25 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPRoyaltiesContext
    
            #region sbrRPSalesAndMarketingContext
            IEnumerable<SBRContext> sbrRPSalesAndMarketingContexts = null;
            SBRContext sbrRPSalesAndMarketingContext = null;
    
            sbrRPSalesAndMarketingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}SalesAndMarketing")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSalesAndMarketingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSalesAndMarketingContexts != null && sbrRPSalesAndMarketingContexts.Count() > 0)
            {
                sbrRPSalesAndMarketingContext = sbrRPSalesAndMarketingContexts.First();
        
                report.RPSalesAndMarketingId = sbrRPSalesAndMarketingContext.Id;
                report.RPSalesAndMarketingIdentifier = sbrRPSalesAndMarketingContext.EntityIdentifier;
                report.RPSalesAndMarketingIdentifierScheme = sbrRPSalesAndMarketingContext.EntityScheme;
        
                report.RPSalesAndMarketingCount = sbrRPSalesAndMarketingContexts.Count();
                report.RPSalesAndMarketingExists = true;
                report.RPSalesAndMarketingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSalesAndMarketingContext) + 1;
                report.RPSalesAndMarketingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSalesAndMarketingContexts.Last()) + 1;
        

                if (report.RPSalesAndMarketingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPSalesAndMarketingIdentifierTFN = report.RPSalesAndMarketingIdentifier;
    
                report.RPSalesAndMarketingStartDate = ((SBRDurationPeriod)sbrRPSalesAndMarketingContext.Period).DurationStartAsDateObject;
                report.RPSalesAndMarketingEndDate = ((SBRDurationPeriod)sbrRPSalesAndMarketingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPSalesAndMarketingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPSalesAndMarketingContexts.Count(); i++)
                {
                    sbrRPSalesAndMarketingContext = sbrRPSalesAndMarketingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSalesAndMarketingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSalesAndMarketingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSalesAndMarketingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSalesAndMarketingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPSalesAndMarketingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS56 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSalesAndMarketingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS57 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSalesAndMarketingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS58 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPSalesAndMarketingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS59 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPSalesAndMarketingContext
    
            #region sbrRPSecuritisationContext
            IEnumerable<SBRContext> sbrRPSecuritisationContexts = null;
            SBRContext sbrRPSecuritisationContext = null;
    
            sbrRPSecuritisationContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Securitisation")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSecuritisationElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSecuritisationContexts != null && sbrRPSecuritisationContexts.Count() > 0)
            {
                sbrRPSecuritisationContext = sbrRPSecuritisationContexts.First();
        
                report.RPSecuritisationId = sbrRPSecuritisationContext.Id;
                report.RPSecuritisationIdentifier = sbrRPSecuritisationContext.EntityIdentifier;
                report.RPSecuritisationIdentifierScheme = sbrRPSecuritisationContext.EntityScheme;
        
                report.RPSecuritisationCount = sbrRPSecuritisationContexts.Count();
                report.RPSecuritisationExists = true;
                report.RPSecuritisationOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSecuritisationContext) + 1;
                report.RPSecuritisationLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSecuritisationContexts.Last()) + 1;
        

                if (report.RPSecuritisationIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPSecuritisationIdentifierTFN = report.RPSecuritisationIdentifier;
    
                report.RPSecuritisationStartDate = ((SBRDurationPeriod)sbrRPSecuritisationContext.Period).DurationStartAsDateObject;
                report.RPSecuritisationEndDate = ((SBRDurationPeriod)sbrRPSecuritisationContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPSecuritisationElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPSecuritisationContexts.Count(); i++)
                {
                    sbrRPSecuritisationContext = sbrRPSecuritisationContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSecuritisationContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSecuritisationElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSecuritisationElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSecuritisationElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPSecuritisationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.BookValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS93 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSecuritisationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Consideration.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS94 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSecuritisationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS95 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPSecuritisationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS96 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPSecuritisationContext
    
            #region sbrRPSoftwareAndITServicesContext
            IEnumerable<SBRContext> sbrRPSoftwareAndITServicesContexts = null;
            SBRContext sbrRPSoftwareAndITServicesContext = null;
    
            sbrRPSoftwareAndITServicesContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}SoftwareAndITServices")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPSoftwareAndITServicesElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPSoftwareAndITServicesContexts != null && sbrRPSoftwareAndITServicesContexts.Count() > 0)
            {
                sbrRPSoftwareAndITServicesContext = sbrRPSoftwareAndITServicesContexts.First();
        
                report.RPSoftwareAndITServicesId = sbrRPSoftwareAndITServicesContext.Id;
                report.RPSoftwareAndITServicesIdentifier = sbrRPSoftwareAndITServicesContext.EntityIdentifier;
                report.RPSoftwareAndITServicesIdentifierScheme = sbrRPSoftwareAndITServicesContext.EntityScheme;
        
                report.RPSoftwareAndITServicesCount = sbrRPSoftwareAndITServicesContexts.Count();
                report.RPSoftwareAndITServicesExists = true;
                report.RPSoftwareAndITServicesOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoftwareAndITServicesContext) + 1;
                report.RPSoftwareAndITServicesLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPSoftwareAndITServicesContexts.Last()) + 1;
        

                if (report.RPSoftwareAndITServicesIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPSoftwareAndITServicesIdentifierTFN = report.RPSoftwareAndITServicesIdentifier;
    
                report.RPSoftwareAndITServicesStartDate = ((SBRDurationPeriod)sbrRPSoftwareAndITServicesContext.Period).DurationStartAsDateObject;
                report.RPSoftwareAndITServicesEndDate = ((SBRDurationPeriod)sbrRPSoftwareAndITServicesContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPSoftwareAndITServicesElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPSoftwareAndITServicesContexts.Count(); i++)
                {
                    sbrRPSoftwareAndITServicesContext = sbrRPSoftwareAndITServicesContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPSoftwareAndITServicesContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPSoftwareAndITServicesElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPSoftwareAndITServicesElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPSoftwareAndITServicesElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPSoftwareAndITServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS60 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSoftwareAndITServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS61 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPSoftwareAndITServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS62 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPSoftwareAndITServicesElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS63 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPSoftwareAndITServicesContext
    
            #region sbrRPTangibleRevenueContext
            IEnumerable<SBRContext> sbrRPTangibleRevenueContexts = null;
            SBRContext sbrRPTangibleRevenueContext = null;
    
            sbrRPTangibleRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TangibleRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTangibleRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTangibleRevenueContexts != null && sbrRPTangibleRevenueContexts.Count() > 0)
            {
                sbrRPTangibleRevenueContext = sbrRPTangibleRevenueContexts.First();
        
                report.RPTangibleRevenueId = sbrRPTangibleRevenueContext.Id;
                report.RPTangibleRevenueIdentifier = sbrRPTangibleRevenueContext.EntityIdentifier;
                report.RPTangibleRevenueIdentifierScheme = sbrRPTangibleRevenueContext.EntityScheme;
        
                report.RPTangibleRevenueCount = sbrRPTangibleRevenueContexts.Count();
                report.RPTangibleRevenueExists = true;
                report.RPTangibleRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTangibleRevenueContext) + 1;
                report.RPTangibleRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTangibleRevenueContexts.Last()) + 1;
        

                if (report.RPTangibleRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTangibleRevenueIdentifierTFN = report.RPTangibleRevenueIdentifier;
    
                report.RPTangibleRevenueStartDate = ((SBRDurationPeriod)sbrRPTangibleRevenueContext.Period).DurationStartAsDateObject;
                report.RPTangibleRevenueEndDate = ((SBRDurationPeriod)sbrRPTangibleRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTangibleRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTangibleRevenueContexts.Count(); i++)
                {
                    sbrRPTangibleRevenueContext = sbrRPTangibleRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTangibleRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTangibleRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTangibleRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTangibleRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTangibleRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS17 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTangibleRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS18 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTangibleRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS19 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPTangibleRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS20 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPTangibleRevenueContext
    
            #region sbrRPTechnicalContext
            IEnumerable<SBRContext> sbrRPTechnicalContexts = null;
            SBRContext sbrRPTechnicalContext = null;
    
            sbrRPTechnicalContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Technical")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTechnicalElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTechnicalContexts != null && sbrRPTechnicalContexts.Count() > 0)
            {
                sbrRPTechnicalContext = sbrRPTechnicalContexts.First();
        
                report.RPTechnicalId = sbrRPTechnicalContext.Id;
                report.RPTechnicalIdentifier = sbrRPTechnicalContext.EntityIdentifier;
                report.RPTechnicalIdentifierScheme = sbrRPTechnicalContext.EntityScheme;
        
                report.RPTechnicalCount = sbrRPTechnicalContexts.Count();
                report.RPTechnicalExists = true;
                report.RPTechnicalOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTechnicalContext) + 1;
                report.RPTechnicalLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTechnicalContexts.Last()) + 1;
        

                if (report.RPTechnicalIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTechnicalIdentifierTFN = report.RPTechnicalIdentifier;
    
                report.RPTechnicalStartDate = ((SBRDurationPeriod)sbrRPTechnicalContext.Period).DurationStartAsDateObject;
                report.RPTechnicalEndDate = ((SBRDurationPeriod)sbrRPTechnicalContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTechnicalElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTechnicalContexts.Count(); i++)
                {
                    sbrRPTechnicalContext = sbrRPTechnicalContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTechnicalContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTechnicalElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTechnicalElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTechnicalElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTechnicalElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS64 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTechnicalElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS65 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTechnicalElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS66 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPTechnicalElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS67 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPTechnicalContext
    
            #region sbrRPTreasuryRelatedContext
            IEnumerable<SBRContext> sbrRPTreasuryRelatedContexts = null;
            SBRContext sbrRPTreasuryRelatedContext = null;
    
            sbrRPTreasuryRelatedContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TreasuryRelated")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTreasuryRelatedElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTreasuryRelatedContexts != null && sbrRPTreasuryRelatedContexts.Count() > 0)
            {
                sbrRPTreasuryRelatedContext = sbrRPTreasuryRelatedContexts.First();
        
                report.RPTreasuryRelatedId = sbrRPTreasuryRelatedContext.Id;
                report.RPTreasuryRelatedIdentifier = sbrRPTreasuryRelatedContext.EntityIdentifier;
                report.RPTreasuryRelatedIdentifierScheme = sbrRPTreasuryRelatedContext.EntityScheme;
        
                report.RPTreasuryRelatedCount = sbrRPTreasuryRelatedContexts.Count();
                report.RPTreasuryRelatedExists = true;
                report.RPTreasuryRelatedOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTreasuryRelatedContext) + 1;
                report.RPTreasuryRelatedLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTreasuryRelatedContexts.Last()) + 1;
        

                if (report.RPTreasuryRelatedIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTreasuryRelatedIdentifierTFN = report.RPTreasuryRelatedIdentifier;
    
                report.RPTreasuryRelatedStartDate = ((SBRDurationPeriod)sbrRPTreasuryRelatedContext.Period).DurationStartAsDateObject;
                report.RPTreasuryRelatedEndDate = ((SBRDurationPeriod)sbrRPTreasuryRelatedContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTreasuryRelatedElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTreasuryRelatedContexts.Count(); i++)
                {
                    sbrRPTreasuryRelatedContext = sbrRPTreasuryRelatedContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTreasuryRelatedContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTreasuryRelatedElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTreasuryRelatedElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTreasuryRelatedElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTreasuryRelatedElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS36 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTreasuryRelatedElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.Total.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS37 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTreasuryRelatedElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ArmsLengthPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS38 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPTreasuryRelatedElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS39 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPTreasuryRelatedContext
    
            #region sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext = null;
    
            sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedAssignmentOfIntangibleNonRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts != null && sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.First();
        
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueId = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.Id;
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueIdentifier = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueIdentifierScheme = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueCount = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.Count();
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueExists = true;
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext) + 1;
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueIdentifierTFN = report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueIdentifier;
    
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext = sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS340 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS341 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.AcquisitionsAndDisposalsPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS342 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfIntangibleNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS343 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedAssignmentOfIntangibleNonRevenueContext
    
            #region sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext = null;
    
            sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts != null && sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.First();
        
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueId = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.Id;
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueIdentifier = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueIdentifierScheme = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueCount = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.Count();
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueExists = true;
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext) + 1;
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueIdentifierTFN = report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueIdentifier;
    
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext = sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS344 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS345 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.AcquisitionsAndDisposalsPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS346 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS347 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueContext
    
            #region sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext
            IEnumerable<SBRContext> sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts = null;
            SBRContext sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext = null;
    
            sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherNotReportedAssignmentOfLoansOrDebtsNonRevenue")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts != null && sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.Count() > 0)
            {
                sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.First();
        
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueId = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.Id;
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueIdentifier = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.EntityIdentifier;
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueIdentifierScheme = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.EntityScheme;
        
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueCount = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.Count();
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueExists = true;
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext) + 1;
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.Last()) + 1;
        

                if (report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueIdentifierTFN = report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueIdentifier;
    
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueStartDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.Period).DurationStartAsDateObject;
                report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueEndDate = ((SBRDurationPeriod)sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.Count(); i++)
                {
                    sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext = sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS348 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.ConsiderationReceived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS349 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.AcquisitionsAndDisposalsPricingMethod.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS350 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS351 = currentValues[0].Value;
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueContext
    
            #region sbrRPCostPlusRemunerationContext
            IEnumerable<SBRContext> sbrRPCostPlusRemunerationContexts = null;
            SBRContext sbrRPCostPlusRemunerationContext = null;
    
            sbrRPCostPlusRemunerationContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}CostPlusRemuneration")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCostPlusRemunerationElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCostPlusRemunerationContexts != null && sbrRPCostPlusRemunerationContexts.Count() > 0)
            {
                sbrRPCostPlusRemunerationContext = sbrRPCostPlusRemunerationContexts.First();
        
                report.RPCostPlusRemunerationId = sbrRPCostPlusRemunerationContext.Id;
                report.RPCostPlusRemunerationIdentifier = sbrRPCostPlusRemunerationContext.EntityIdentifier;
                report.RPCostPlusRemunerationIdentifierScheme = sbrRPCostPlusRemunerationContext.EntityScheme;
        
                report.RPCostPlusRemunerationCount = sbrRPCostPlusRemunerationContexts.Count();
                report.RPCostPlusRemunerationExists = true;
                report.RPCostPlusRemunerationOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCostPlusRemunerationContext) + 1;
                report.RPCostPlusRemunerationLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCostPlusRemunerationContexts.Last()) + 1;
        

                if (report.RPCostPlusRemunerationIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCostPlusRemunerationIdentifierTFN = report.RPCostPlusRemunerationIdentifier;
    
                report.RPCostPlusRemunerationStartDate = ((SBRDurationPeriod)sbrRPCostPlusRemunerationContext.Period).DurationStartAsDateObject;
                report.RPCostPlusRemunerationEndDate = ((SBRDurationPeriod)sbrRPCostPlusRemunerationContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCostPlusRemunerationElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCostPlusRemunerationContexts.Count(); i++)
                {
                    sbrRPCostPlusRemunerationContext = sbrRPCostPlusRemunerationContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCostPlusRemunerationContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCostPlusRemunerationElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCostPlusRemunerationElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCostPlusRemunerationElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCostPlusRemunerationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.ResearchAndDevelopment.CostPlusRemuneration.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS427 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCostPlusRemunerationElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ResearchAndDevelopment.CostPlusRemuneration.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS428 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPCostPlusRemunerationContext
    
            #region sbrRPTrustExemptionCodeContext
            IEnumerable<SBRContext> sbrRPTrustExemptionCodeContexts = null;
            SBRContext sbrRPTrustExemptionCodeContext = null;
    
            sbrRPTrustExemptionCodeContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}TransferorTrustExemptionCodeDimension")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTrustExemptionCodeElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTrustExemptionCodeContexts != null && sbrRPTrustExemptionCodeContexts.Count() > 0)
            {
        
                report.RPTrustExemptionCodeContextCollection = new List<IDS2021.RPTrustExemptionCodeContext>();
                for (int i = 0; i < sbrRPTrustExemptionCodeContexts.Count(); i++)
                {
                    sbrRPTrustExemptionCodeContext = sbrRPTrustExemptionCodeContexts.ElementAt(i);
                    IDS2021.RPTrustExemptionCodeContext rpTrustExemptionCodeContext = new IDS2021.RPTrustExemptionCodeContext();
                    report.RPTrustExemptionCodeContextCollection.Add(rpTrustExemptionCodeContext);
            
                    rpTrustExemptionCodeContext.Id = sbrRPTrustExemptionCodeContext.Id;
                    rpTrustExemptionCodeContext.Identifier = sbrRPTrustExemptionCodeContext.EntityIdentifier;
                    rpTrustExemptionCodeContext.IdentifierScheme = sbrRPTrustExemptionCodeContext.EntityScheme;
            
                    rpTrustExemptionCodeContext.Count = sbrRPTrustExemptionCodeContexts.Count();
                    rpTrustExemptionCodeContext.Exists = true;
                    rpTrustExemptionCodeContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrustExemptionCodeContext) + 1;
                    rpTrustExemptionCodeContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTrustExemptionCodeContexts.Last()) + 1;
            
    
                    if (rpTrustExemptionCodeContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpTrustExemptionCodeContext.IdentifierTFN = rpTrustExemptionCodeContext.Identifier;
        
                    rpTrustExemptionCodeContext.StartDate = ((SBRDurationPeriod)sbrRPTrustExemptionCodeContext.Period).DurationStartAsDateObject;
                    rpTrustExemptionCodeContext.EndDate = ((SBRDurationPeriod)sbrRPTrustExemptionCodeContext.Period).DurationEndAsDateObject;
            
                    currentSegment = sbrRPTrustExemptionCodeContext.Segment.Segment;
            
                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");
            
                    currentNode = currentSegment.SelectSingleNode("xbrldi:explicitMember[contains(@dimension, 'TransferorTrustExemptionCodeDimension')]", currentNamespaceManager);
            
                    if (currentNode != null)
                    {
                        rpTrustExemptionCodeContext.TransferorTrustExemptionCodeDimensionValue = currentNode.InnerText.Substring(currentNode.InnerText.IndexOf(":") + 1).Trim();
                    }
            
                    sbrPopulatedRPTrustExemptionCodeElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTrustExemptionCodeContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTrustExemptionCodeElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTrustExemptionCodeElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTrustExemptionCodeElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
                    rpTrustExemptionCodeContext.IDS215 = rpTrustExemptionCodeContext.TransferorTrustExemptionCodeDimensionValue;
            
                    if (sbrPopulatedRPTrustExemptionCodeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.TransferPropertyMoneyOrServices.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpTrustExemptionCodeContext.IDS214 = decimal.Parse(currentValues[0].Value);
                    }
        } 
            } 
            #endregion End of Context sbrRPTrustExemptionCodeContext
    
            #region sbrRPAdvisoryContext
            IEnumerable<SBRContext> sbrRPAdvisoryContexts = null;
            SBRContext sbrRPAdvisoryContext = null;
    
            sbrRPAdvisoryContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Advisory")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPAdvisoryElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPAdvisoryContexts != null && sbrRPAdvisoryContexts.Count() > 0)
            {
                sbrRPAdvisoryContext = sbrRPAdvisoryContexts.First();
        
                report.RPAdvisoryId = sbrRPAdvisoryContext.Id;
                report.RPAdvisoryIdentifier = sbrRPAdvisoryContext.EntityIdentifier;
                report.RPAdvisoryIdentifierScheme = sbrRPAdvisoryContext.EntityScheme;
        
                report.RPAdvisoryCount = sbrRPAdvisoryContexts.Count();
                report.RPAdvisoryExists = true;
                report.RPAdvisoryOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAdvisoryContext) + 1;
                report.RPAdvisoryLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAdvisoryContexts.Last()) + 1;
        

                if (report.RPAdvisoryIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPAdvisoryIdentifierTFN = report.RPAdvisoryIdentifier;
    
                report.RPAdvisoryStartDate = ((SBRDurationPeriod)sbrRPAdvisoryContext.Period).DurationStartAsDateObject;
                report.RPAdvisoryEndDate = ((SBRDurationPeriod)sbrRPAdvisoryContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPAdvisoryElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPAdvisoryContexts.Count(); i++)
                {
                    sbrRPAdvisoryContext = sbrRPAdvisoryContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPAdvisoryContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPAdvisoryElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPAdvisoryElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPAdvisoryElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPAdvisoryElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS287 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPAdvisoryElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS288 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPAdvisoryContext
    
            #region sbrRPBorrowingContext
            IEnumerable<SBRContext> sbrRPBorrowingContexts = null;
            SBRContext sbrRPBorrowingContext = null;
    
            sbrRPBorrowingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Borrowing")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBorrowingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBorrowingContexts != null && sbrRPBorrowingContexts.Count() > 0)
            {
                sbrRPBorrowingContext = sbrRPBorrowingContexts.First();
        
                report.RPBorrowingId = sbrRPBorrowingContext.Id;
                report.RPBorrowingIdentifier = sbrRPBorrowingContext.EntityIdentifier;
                report.RPBorrowingIdentifierScheme = sbrRPBorrowingContext.EntityScheme;
        
                report.RPBorrowingCount = sbrRPBorrowingContexts.Count();
                report.RPBorrowingExists = true;
                report.RPBorrowingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBorrowingContext) + 1;
                report.RPBorrowingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBorrowingContexts.Last()) + 1;
        

                if (report.RPBorrowingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPBorrowingIdentifierTFN = report.RPBorrowingIdentifier;
    
                report.RPBorrowingStartDate = ((SBRDurationPeriod)sbrRPBorrowingContext.Period).DurationStartAsDateObject;
                report.RPBorrowingEndDate = ((SBRDurationPeriod)sbrRPBorrowingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPBorrowingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPBorrowingContexts.Count(); i++)
                {
                    sbrRPBorrowingContext = sbrRPBorrowingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBorrowingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBorrowingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBorrowingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBorrowingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPBorrowingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS269 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBorrowingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS270 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPBorrowingContext
    
            #region sbrRPDivision770Context
            IEnumerable<SBRContext> sbrRPDivision770Contexts = null;
            SBRContext sbrRPDivision770Context = null;
    
            sbrRPDivision770Contexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Division770")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPDivision770ElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPDivision770Contexts != null && sbrRPDivision770Contexts.Count() > 0)
            {
                sbrRPDivision770Context = sbrRPDivision770Contexts.First();
        
                report.RPDivision770Id = sbrRPDivision770Context.Id;
                report.RPDivision770Identifier = sbrRPDivision770Context.EntityIdentifier;
                report.RPDivision770IdentifierScheme = sbrRPDivision770Context.EntityScheme;
        
                report.RPDivision770Count = sbrRPDivision770Contexts.Count();
                report.RPDivision770Exists = true;
                report.RPDivision770OccurrenceIndex = Array.IndexOf(contextArray, sbrRPDivision770Context) + 1;
                report.RPDivision770LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDivision770Contexts.Last()) + 1;
        

                if (report.RPDivision770IdentifierScheme == "http://www.ato.gov.au/tfn") report.RPDivision770IdentifierTFN = report.RPDivision770Identifier;
    
                report.RPDivision770StartDate = ((SBRDurationPeriod)sbrRPDivision770Context.Period).DurationStartAsDateObject;
                report.RPDivision770EndDate = ((SBRDurationPeriod)sbrRPDivision770Context.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPDivision770ElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPDivision770Contexts.Count(); i++)
                {
                    sbrRPDivision770Context = sbrRPDivision770Contexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPDivision770Context.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPDivision770ElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPDivision770ElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPDivision770ElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPDivision770ElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeForeignIncomeTaxEligibleFractionClaimed.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS306 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPDivision770Context
    
            #region sbrRPEligibleContractContext
            IEnumerable<SBRContext> sbrRPEligibleContractContexts = null;
            SBRContext sbrRPEligibleContractContext = null;
    
            sbrRPEligibleContractContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}EligibleContract")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPEligibleContractElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPEligibleContractContexts != null && sbrRPEligibleContractContexts.Count() > 0)
            {
                sbrRPEligibleContractContext = sbrRPEligibleContractContexts.First();
        
                report.RPEligibleContractId = sbrRPEligibleContractContext.Id;
                report.RPEligibleContractIdentifier = sbrRPEligibleContractContext.EntityIdentifier;
                report.RPEligibleContractIdentifierScheme = sbrRPEligibleContractContext.EntityScheme;
        
                report.RPEligibleContractCount = sbrRPEligibleContractContexts.Count();
                report.RPEligibleContractExists = true;
                report.RPEligibleContractOccurrenceIndex = Array.IndexOf(contextArray, sbrRPEligibleContractContext) + 1;
                report.RPEligibleContractLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPEligibleContractContexts.Last()) + 1;
        

                if (report.RPEligibleContractIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPEligibleContractIdentifierTFN = report.RPEligibleContractIdentifier;
    
                report.RPEligibleContractStartDate = ((SBRDurationPeriod)sbrRPEligibleContractContext.Period).DurationStartAsDateObject;
                report.RPEligibleContractEndDate = ((SBRDurationPeriod)sbrRPEligibleContractContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPEligibleContractElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPEligibleContractContexts.Count(); i++)
                {
                    sbrRPEligibleContractContext = sbrRPEligibleContractContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPEligibleContractContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPEligibleContractElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPEligibleContractElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPEligibleContractElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPEligibleContractElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS281 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPEligibleContractElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS282 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPEligibleContractContext
    
            #region sbrRPGuaranteeTypeContext
            IEnumerable<SBRContext> sbrRPGuaranteeTypeContexts = null;
            SBRContext sbrRPGuaranteeTypeContext = null;
    
            sbrRPGuaranteeTypeContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}GuaranteeType")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPGuaranteeTypeElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPGuaranteeTypeContexts != null && sbrRPGuaranteeTypeContexts.Count() > 0)
            {
                sbrRPGuaranteeTypeContext = sbrRPGuaranteeTypeContexts.First();
        
                report.RPGuaranteeTypeId = sbrRPGuaranteeTypeContext.Id;
                report.RPGuaranteeTypeIdentifier = sbrRPGuaranteeTypeContext.EntityIdentifier;
                report.RPGuaranteeTypeIdentifierScheme = sbrRPGuaranteeTypeContext.EntityScheme;
        
                report.RPGuaranteeTypeCount = sbrRPGuaranteeTypeContexts.Count();
                report.RPGuaranteeTypeExists = true;
                report.RPGuaranteeTypeOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGuaranteeTypeContext) + 1;
                report.RPGuaranteeTypeLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGuaranteeTypeContexts.Last()) + 1;
        

                if (report.RPGuaranteeTypeIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPGuaranteeTypeIdentifierTFN = report.RPGuaranteeTypeIdentifier;
    
                report.RPGuaranteeTypeStartDate = ((SBRDurationPeriod)sbrRPGuaranteeTypeContext.Period).DurationStartAsDateObject;
                report.RPGuaranteeTypeEndDate = ((SBRDurationPeriod)sbrRPGuaranteeTypeContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPGuaranteeTypeElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPGuaranteeTypeContexts.Count(); i++)
                {
                    sbrRPGuaranteeTypeContext = sbrRPGuaranteeTypeContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPGuaranteeTypeContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPGuaranteeTypeElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPGuaranteeTypeElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPGuaranteeTypeElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPGuaranteeTypeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS275 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPGuaranteeTypeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS276 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPGuaranteeTypeContext
    
            #region sbrRPGeneralDeductionContext
            IEnumerable<SBRContext> sbrRPGeneralDeductionContexts = null;
            SBRContext sbrRPGeneralDeductionContext = null;
    
            sbrRPGeneralDeductionContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}GeneralDeduction")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPGeneralDeductionElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPGeneralDeductionContexts != null && sbrRPGeneralDeductionContexts.Count() > 0)
            {
                sbrRPGeneralDeductionContext = sbrRPGeneralDeductionContexts.First();
        
                report.RPGeneralDeductionId = sbrRPGeneralDeductionContext.Id;
                report.RPGeneralDeductionIdentifier = sbrRPGeneralDeductionContext.EntityIdentifier;
                report.RPGeneralDeductionIdentifierScheme = sbrRPGeneralDeductionContext.EntityScheme;
        
                report.RPGeneralDeductionCount = sbrRPGeneralDeductionContexts.Count();
                report.RPGeneralDeductionExists = true;
                report.RPGeneralDeductionOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGeneralDeductionContext) + 1;
                report.RPGeneralDeductionLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGeneralDeductionContexts.Last()) + 1;
        

                if (report.RPGeneralDeductionIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPGeneralDeductionIdentifierTFN = report.RPGeneralDeductionIdentifier;
    
                report.RPGeneralDeductionStartDate = ((SBRDurationPeriod)sbrRPGeneralDeductionContext.Period).DurationStartAsDateObject;
                report.RPGeneralDeductionEndDate = ((SBRDurationPeriod)sbrRPGeneralDeductionContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPGeneralDeductionElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPGeneralDeductionContexts.Count(); i++)
                {
                    sbrRPGeneralDeductionContext = sbrRPGeneralDeductionContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPGeneralDeductionContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPGeneralDeductionElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPGeneralDeductionElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPGeneralDeductionElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPGeneralDeductionElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.GeneralOffshoreBankingDeductionAndApportionableOffshoreBankingDeduction.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS296 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPGeneralDeductionContext
    
            #region sbrRPHedgingContext
            IEnumerable<SBRContext> sbrRPHedgingContexts = null;
            SBRContext sbrRPHedgingContext = null;
    
            sbrRPHedgingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Hedging")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPHedgingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPHedgingContexts != null && sbrRPHedgingContexts.Count() > 0)
            {
                sbrRPHedgingContext = sbrRPHedgingContexts.First();
        
                report.RPHedgingId = sbrRPHedgingContext.Id;
                report.RPHedgingIdentifier = sbrRPHedgingContext.EntityIdentifier;
                report.RPHedgingIdentifierScheme = sbrRPHedgingContext.EntityScheme;
        
                report.RPHedgingCount = sbrRPHedgingContexts.Count();
                report.RPHedgingExists = true;
                report.RPHedgingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHedgingContext) + 1;
                report.RPHedgingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHedgingContexts.Last()) + 1;
        

                if (report.RPHedgingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPHedgingIdentifierTFN = report.RPHedgingIdentifier;
    
                report.RPHedgingStartDate = ((SBRDurationPeriod)sbrRPHedgingContext.Period).DurationStartAsDateObject;
                report.RPHedgingEndDate = ((SBRDurationPeriod)sbrRPHedgingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPHedgingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPHedgingContexts.Count(); i++)
                {
                    sbrRPHedgingContext = sbrRPHedgingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPHedgingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPHedgingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPHedgingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPHedgingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPHedgingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS290 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPHedgingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS291 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPHedgingContext
    
            #region sbrRPInvestmentContext
            IEnumerable<SBRContext> sbrRPInvestmentContexts = null;
            SBRContext sbrRPInvestmentContext = null;
    
            sbrRPInvestmentContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Investment")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInvestmentElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInvestmentContexts != null && sbrRPInvestmentContexts.Count() > 0)
            {
                sbrRPInvestmentContext = sbrRPInvestmentContexts.First();
        
                report.RPInvestmentId = sbrRPInvestmentContext.Id;
                report.RPInvestmentIdentifier = sbrRPInvestmentContext.EntityIdentifier;
                report.RPInvestmentIdentifierScheme = sbrRPInvestmentContext.EntityScheme;
        
                report.RPInvestmentCount = sbrRPInvestmentContexts.Count();
                report.RPInvestmentExists = true;
                report.RPInvestmentOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestmentContext) + 1;
                report.RPInvestmentLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestmentContexts.Last()) + 1;
        

                if (report.RPInvestmentIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInvestmentIdentifierTFN = report.RPInvestmentIdentifier;
    
                report.RPInvestmentStartDate = ((SBRDurationPeriod)sbrRPInvestmentContext.Period).DurationStartAsDateObject;
                report.RPInvestmentEndDate = ((SBRDurationPeriod)sbrRPInvestmentContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPInvestmentElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInvestmentContexts.Count(); i++)
                {
                    sbrRPInvestmentContext = sbrRPInvestmentContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInvestmentContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInvestmentElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInvestmentElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInvestmentElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPInvestmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS284 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPInvestmentElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS285 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPInvestmentContext
    
            #region sbrRPLendingContext
            IEnumerable<SBRContext> sbrRPLendingContexts = null;
            SBRContext sbrRPLendingContext = null;
    
            sbrRPLendingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Lending")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLendingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLendingContexts != null && sbrRPLendingContexts.Count() > 0)
            {
                sbrRPLendingContext = sbrRPLendingContexts.First();
        
                report.RPLendingId = sbrRPLendingContext.Id;
                report.RPLendingIdentifier = sbrRPLendingContext.EntityIdentifier;
                report.RPLendingIdentifierScheme = sbrRPLendingContext.EntityScheme;
        
                report.RPLendingCount = sbrRPLendingContexts.Count();
                report.RPLendingExists = true;
                report.RPLendingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLendingContext) + 1;
                report.RPLendingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLendingContexts.Last()) + 1;
        

                if (report.RPLendingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLendingIdentifierTFN = report.RPLendingIdentifier;
    
                report.RPLendingStartDate = ((SBRDurationPeriod)sbrRPLendingContext.Period).DurationStartAsDateObject;
                report.RPLendingEndDate = ((SBRDurationPeriod)sbrRPLendingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLendingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLendingContexts.Count(); i++)
                {
                    sbrRPLendingContext = sbrRPLendingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLendingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLendingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLendingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLendingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLendingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS272 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLendingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS273 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPLendingContext
    
            #region sbrRPOtherContext
            IEnumerable<SBRContext> sbrRPOtherContexts = null;
            SBRContext sbrRPOtherContext = null;
    
            sbrRPOtherContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Other")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherContexts != null && sbrRPOtherContexts.Count() > 0)
            {
                sbrRPOtherContext = sbrRPOtherContexts.First();
        
                report.RPOtherId = sbrRPOtherContext.Id;
                report.RPOtherIdentifier = sbrRPOtherContext.EntityIdentifier;
                report.RPOtherIdentifierScheme = sbrRPOtherContext.EntityScheme;
        
                report.RPOtherCount = sbrRPOtherContexts.Count();
                report.RPOtherExists = true;
                report.RPOtherOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherContext) + 1;
                report.RPOtherLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherContexts.Last()) + 1;
        

                if (report.RPOtherIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherIdentifierTFN = report.RPOtherIdentifier;
    
                report.RPOtherStartDate = ((SBRDurationPeriod)sbrRPOtherContext.Period).DurationStartAsDateObject;
                report.RPOtherEndDate = ((SBRDurationPeriod)sbrRPOtherContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherContexts.Count(); i++)
                {
                    sbrRPOtherContext = sbrRPOtherContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS293 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS294 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherContext
    
            #region sbrRPTotalNetContext
            IEnumerable<SBRContext> sbrRPTotalNetContexts = null;
            SBRContext sbrRPTotalNetContext = null;
    
            sbrRPTotalNetContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TotalNet")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTotalNetElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTotalNetContexts != null && sbrRPTotalNetContexts.Count() > 0)
            {
                sbrRPTotalNetContext = sbrRPTotalNetContexts.First();
        
                report.RPTotalNetId = sbrRPTotalNetContext.Id;
                report.RPTotalNetIdentifier = sbrRPTotalNetContext.EntityIdentifier;
                report.RPTotalNetIdentifierScheme = sbrRPTotalNetContext.EntityScheme;
        
                report.RPTotalNetCount = sbrRPTotalNetContexts.Count();
                report.RPTotalNetExists = true;
                report.RPTotalNetOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTotalNetContext) + 1;
                report.RPTotalNetLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTotalNetContexts.Last()) + 1;
        

                if (report.RPTotalNetIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTotalNetIdentifierTFN = report.RPTotalNetIdentifier;
    
                report.RPTotalNetStartDate = ((SBRDurationPeriod)sbrRPTotalNetContext.Period).DurationStartAsDateObject;
                report.RPTotalNetEndDate = ((SBRDurationPeriod)sbrRPTotalNetContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTotalNetElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTotalNetContexts.Count(); i++)
                {
                    sbrRPTotalNetContext = sbrRPTotalNetContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTotalNetContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTotalNetElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTotalNetElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTotalNetElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTotalNetElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS303 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTotalNetElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS304 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPTotalNetContext
    
            #region sbrRPTradingContext
            IEnumerable<SBRContext> sbrRPTradingContexts = null;
            SBRContext sbrRPTradingContext = null;
    
            sbrRPTradingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Trading")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTradingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTradingContexts != null && sbrRPTradingContexts.Count() > 0)
            {
                sbrRPTradingContext = sbrRPTradingContexts.First();
        
                report.RPTradingId = sbrRPTradingContext.Id;
                report.RPTradingIdentifier = sbrRPTradingContext.EntityIdentifier;
                report.RPTradingIdentifierScheme = sbrRPTradingContext.EntityScheme;
        
                report.RPTradingCount = sbrRPTradingContexts.Count();
                report.RPTradingExists = true;
                report.RPTradingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTradingContext) + 1;
                report.RPTradingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTradingContexts.Last()) + 1;
        

                if (report.RPTradingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTradingIdentifierTFN = report.RPTradingIdentifier;
    
                report.RPTradingStartDate = ((SBRDurationPeriod)sbrRPTradingContext.Period).DurationStartAsDateObject;
                report.RPTradingEndDate = ((SBRDurationPeriod)sbrRPTradingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTradingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTradingContexts.Count(); i++)
                {
                    sbrRPTradingContext = sbrRPTradingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTradingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTradingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTradingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTradingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTradingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS278 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTradingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS279 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPTradingContext
    
            #region sbrRPLeasingContext
            IEnumerable<SBRContext> sbrRPLeasingContexts = null;
            SBRContext sbrRPLeasingContext = null;
    
            sbrRPLeasingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Leasing")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLeasingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLeasingContexts != null && sbrRPLeasingContexts.Count() > 0)
            {
                sbrRPLeasingContext = sbrRPLeasingContexts.First();
        
                report.RPLeasingId = sbrRPLeasingContext.Id;
                report.RPLeasingIdentifier = sbrRPLeasingContext.EntityIdentifier;
                report.RPLeasingIdentifierScheme = sbrRPLeasingContext.EntityScheme;
        
                report.RPLeasingCount = sbrRPLeasingContexts.Count();
                report.RPLeasingExists = true;
                report.RPLeasingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLeasingContext) + 1;
                report.RPLeasingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLeasingContexts.Last()) + 1;
        

                if (report.RPLeasingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLeasingIdentifierTFN = report.RPLeasingIdentifier;
    
                report.RPLeasingStartDate = ((SBRDurationPeriod)sbrRPLeasingContext.Period).DurationStartAsDateObject;
                report.RPLeasingEndDate = ((SBRDurationPeriod)sbrRPLeasingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLeasingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLeasingContexts.Count(); i++)
                {
                    sbrRPLeasingContext = sbrRPLeasingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLeasingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLeasingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLeasingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLeasingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLeasingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS326 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPLeasingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeOrLossNet.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS327 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPLeasingContext
    
            #region sbrRPCFCLCContext
            IEnumerable<SBRContext> sbrRPCFCLCContexts = null;
            SBRContext sbrRPCFCLCContext = null;
    
            sbrRPCFCLCContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CountryOfResidencyCFCClassificationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ListedCountry")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCFCLCElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCFCLCContexts != null && sbrRPCFCLCContexts.Count() > 0)
            {
                sbrRPCFCLCContext = sbrRPCFCLCContexts.First();
        
                report.RPCFCLCId = sbrRPCFCLCContext.Id;
                report.RPCFCLCIdentifier = sbrRPCFCLCContext.EntityIdentifier;
                report.RPCFCLCIdentifierScheme = sbrRPCFCLCContext.EntityScheme;
        
                report.RPCFCLCCount = sbrRPCFCLCContexts.Count();
                report.RPCFCLCExists = true;
                report.RPCFCLCOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCLCContext) + 1;
                report.RPCFCLCLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCLCContexts.Last()) + 1;
        

                if (report.RPCFCLCIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCFCLCIdentifierTFN = report.RPCFCLCIdentifier;
    
                report.RPCFCLCStartDate = ((SBRDurationPeriod)sbrRPCFCLCContext.Period).DurationStartAsDateObject;
                report.RPCFCLCEndDate = ((SBRDurationPeriod)sbrRPCFCLCContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCFCLCElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCFCLCContexts.Count(); i++)
                {
                    sbrRPCFCLCContext = sbrRPCFCLCContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCFCLCContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCFCLCElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCFCLCElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCFCLCElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS185 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestAcquired.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS368 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestDisposed.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS372 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS375 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS436 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyExcludedTaintedInterestIncome.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS440 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAttributableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS188 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS443 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS447 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAdjustedTaintedConcessionIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS378 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAdjustedTaintedIncomeNotDerived.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS379 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesOtherNotionalAssessableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS380 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAssessableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS381 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS451 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignBranchProfitsOfAustralianCompaniesNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS196 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.PaidOutOfAttributedControlledForeignCompanyIncomeNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS200 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCLCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.NonPortfolioDividendFromForeignCompanyNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS320 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPCFCLCContext
    
            #region sbrRPCFCSPContext
            IEnumerable<SBRContext> sbrRPCFCSPContexts = null;
            SBRContext sbrRPCFCSPContext = null;
    
            sbrRPCFCSPContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CountryOfResidencyCFCClassificationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}SpecifiedCountry")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCFCSPElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCFCSPContexts != null && sbrRPCFCSPContexts.Count() > 0)
            {
                sbrRPCFCSPContext = sbrRPCFCSPContexts.First();
        
                report.RPCFCSPId = sbrRPCFCSPContext.Id;
                report.RPCFCSPIdentifier = sbrRPCFCSPContext.EntityIdentifier;
                report.RPCFCSPIdentifierScheme = sbrRPCFCSPContext.EntityScheme;
        
                report.RPCFCSPCount = sbrRPCFCSPContexts.Count();
                report.RPCFCSPExists = true;
                report.RPCFCSPOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCSPContext) + 1;
                report.RPCFCSPLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCSPContexts.Last()) + 1;
        

                if (report.RPCFCSPIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCFCSPIdentifierTFN = report.RPCFCSPIdentifier;
    
                report.RPCFCSPStartDate = ((SBRDurationPeriod)sbrRPCFCSPContext.Period).DurationStartAsDateObject;
                report.RPCFCSPEndDate = ((SBRDurationPeriod)sbrRPCFCSPContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCFCSPElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCFCSPContexts.Count(); i++)
                {
                    sbrRPCFCSPContext = sbrRPCFCSPContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCFCSPContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCFCSPElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCFCSPElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCFCSPElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS186 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestAcquired.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS369 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestDisposed.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS373 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS376 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS437 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyExcludedTaintedInterestIncome.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS441 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAttributableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS189 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS444 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS448 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAdjustedTaintedIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS382 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesOtherNotionalAssessableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS383 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAssessableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS384 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS452 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignBranchProfitsOfAustralianCompaniesNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS197 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.PaidOutOfAttributedControlledForeignCompanyIncomeNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS201 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCSPElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.NonPortfolioDividendFromForeignCompanyNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS321 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPCFCSPContext
    
            #region sbrRPCFCUCContext
            IEnumerable<SBRContext> sbrRPCFCUCContexts = null;
            SBRContext sbrRPCFCUCContext = null;
    
            sbrRPCFCUCContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CountryOfResidencyCFCClassificationDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}UnlistedCountry")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPCFCUCElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPCFCUCContexts != null && sbrRPCFCUCContexts.Count() > 0)
            {
                sbrRPCFCUCContext = sbrRPCFCUCContexts.First();
        
                report.RPCFCUCId = sbrRPCFCUCContext.Id;
                report.RPCFCUCIdentifier = sbrRPCFCUCContext.EntityIdentifier;
                report.RPCFCUCIdentifierScheme = sbrRPCFCUCContext.EntityScheme;
        
                report.RPCFCUCCount = sbrRPCFCUCContexts.Count();
                report.RPCFCUCExists = true;
                report.RPCFCUCOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCUCContext) + 1;
                report.RPCFCUCLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPCFCUCContexts.Last()) + 1;
        

                if (report.RPCFCUCIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPCFCUCIdentifierTFN = report.RPCFCUCIdentifier;
    
                report.RPCFCUCStartDate = ((SBRDurationPeriod)sbrRPCFCUCContext.Period).DurationStartAsDateObject;
                report.RPCFCUCEndDate = ((SBRDurationPeriod)sbrRPCFCUCContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPCFCUCElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPCFCUCContexts.Count(); i++)
                {
                    sbrRPCFCUCContext = sbrRPCFCUCContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPCFCUCContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPCFCUCElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPCFCUCElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPCFCUCElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesTrustsInterest.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS187 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestAcquired.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS370 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyTrustInterestDisposed.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS374 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS377 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS438 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.ControlledForeignCompanyExcludedTaintedInterestIncome.Count", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS442 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAttributableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS190 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS445 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS449 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesAdjustedTaintedIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS385 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesOtherNotionalAssessableIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS386 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAssessableIncomeTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS387 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS453 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.ForeignBranchProfitsOfAustralianCompaniesNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS198 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.PaidOutOfAttributedControlledForeignCompanyIncomeNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS202 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPCFCUCElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.NonPortfolioDividendFromForeignCompanyNonAssessableNonExemptIncome.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS322 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPCFCUCContext
    
            #region sbrRPRestructuringEventContext
            IEnumerable<SBRContext> sbrRPRestructuringEventContexts = null;
            SBRContext sbrRPRestructuringEventContext = null;
    
            sbrRPRestructuringEventContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.TYPED && dd.Dimension.EndsWith("}RestructuringEventDimension")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPRestructuringEventElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPRestructuringEventContexts != null && sbrRPRestructuringEventContexts.Count() > 0)
            {
        
                report.RPRestructuringEventContextCollection = new List<IDS2021.RPRestructuringEventContext>();
                for (int i = 0; i < sbrRPRestructuringEventContexts.Count(); i++)
                {
                    sbrRPRestructuringEventContext = sbrRPRestructuringEventContexts.ElementAt(i);
                    IDS2021.RPRestructuringEventContext rpRestructuringEventContext = new IDS2021.RPRestructuringEventContext();
                    report.RPRestructuringEventContextCollection.Add(rpRestructuringEventContext);
            
                    rpRestructuringEventContext.Id = sbrRPRestructuringEventContext.Id;
                    rpRestructuringEventContext.Identifier = sbrRPRestructuringEventContext.EntityIdentifier;
                    rpRestructuringEventContext.IdentifierScheme = sbrRPRestructuringEventContext.EntityScheme;
            
                    rpRestructuringEventContext.Count = sbrRPRestructuringEventContexts.Count();
                    rpRestructuringEventContext.Exists = true;
                    rpRestructuringEventContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPRestructuringEventContext) + 1;
                    rpRestructuringEventContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPRestructuringEventContexts.Last()) + 1;
            
    
                    if (rpRestructuringEventContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpRestructuringEventContext.IdentifierTFN = rpRestructuringEventContext.Identifier;
        
                    rpRestructuringEventContext.StartDate = ((SBRDurationPeriod)sbrRPRestructuringEventContext.Period).DurationStartAsDateObject;
                    rpRestructuringEventContext.EndDate = ((SBRDurationPeriod)sbrRPRestructuringEventContext.Period).DurationEndAsDateObject;
            
                    currentSegment = sbrRPRestructuringEventContext.Segment.Segment;
            
                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");
            
                    currentNode = currentSegment.SelectSingleNode("xbrldi:typedMember[contains(@dimension, 'RestructuringEventDimension')]", currentNamespaceManager);
            
                    if (currentNode != null)
                    {
                        rpRestructuringEventContext.RestructuringEventDimensionValue = currentNode.InnerText.Trim();
                    }
            
                    sbrPopulatedRPRestructuringEventElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPRestructuringEventContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPRestructuringEventElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPRestructuringEventElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPRestructuringEventElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
                    rpRestructuringEventContext.IDS157 = rpRestructuringEventContext.RestructuringEventDimensionValue;
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.RestructuringEventEntity.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS152 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.CapitalValue.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS153 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.NatureOfItem.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS154 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}OrganisationDetails.InternationalRelatedPartyCountry.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS155 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.PercentageOfDealingsWithDocumentation.Code", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS156 = currentValues[0].Value;
                    }
            
                    if (sbrPopulatedRPRestructuringEventElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.EventAnalysis.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpRestructuringEventContext.IDS158 = ToBoolean(currentValues[0].Value);
                    }
        } 
            } 
            #endregion End of Context sbrRPRestructuringEventContext
    
            #region sbrRPDTAContext
            IEnumerable<SBRContext> sbrRPDTAContexts = null;
            SBRContext sbrRPDTAContext = null;
    
            sbrRPDTAContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}WithholdingTaxExemptionDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}DoubleTaxAgreement")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPDTAElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPDTAContexts != null && sbrRPDTAContexts.Count() > 0)
            {
                sbrRPDTAContext = sbrRPDTAContexts.First();
        
                report.RPDTAId = sbrRPDTAContext.Id;
                report.RPDTAIdentifier = sbrRPDTAContext.EntityIdentifier;
                report.RPDTAIdentifierScheme = sbrRPDTAContext.EntityScheme;
        
                report.RPDTACount = sbrRPDTAContexts.Count();
                report.RPDTAExists = true;
                report.RPDTAOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDTAContext) + 1;
                report.RPDTALastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPDTAContexts.Last()) + 1;
        

                if (report.RPDTAIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPDTAIdentifierTFN = report.RPDTAIdentifier;
    
                report.RPDTAStartDate = ((SBRDurationPeriod)sbrRPDTAContext.Period).DurationStartAsDateObject;
                report.RPDTAEndDate = ((SBRDurationPeriod)sbrRPDTAContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPDTAElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPDTAContexts.Count(); i++)
                {
                    sbrRPDTAContext = sbrRPDTAContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPDTAContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPDTAElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPDTAElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPDTAElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPDTAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.WithholdingTaxExemptNonResidentInterestPaid.Indicator", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS315 = ToBoolean(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPDTAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.WithholdingTaxExemptNonResidentInterestPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS316 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPDTAContext
    
            #region sbrRPs128FContext
            IEnumerable<SBRContext> sbrRPs128FContexts = null;
            SBRContext sbrRPs128FContext = null;
    
            sbrRPs128FContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}WithholdingTaxExemptionDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Section128F")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPs128FElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPs128FContexts != null && sbrRPs128FContexts.Count() > 0)
            {
                sbrRPs128FContext = sbrRPs128FContexts.First();
        
                report.RPs128FId = sbrRPs128FContext.Id;
                report.RPs128FIdentifier = sbrRPs128FContext.EntityIdentifier;
                report.RPs128FIdentifierScheme = sbrRPs128FContext.EntityScheme;
        
                report.RPs128FCount = sbrRPs128FContexts.Count();
                report.RPs128FExists = true;
                report.RPs128FOccurrenceIndex = Array.IndexOf(contextArray, sbrRPs128FContext) + 1;
                report.RPs128FLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPs128FContexts.Last()) + 1;
        

                if (report.RPs128FIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPs128FIdentifierTFN = report.RPs128FIdentifier;
    
                report.RPs128FStartDate = ((SBRDurationPeriod)sbrRPs128FContext.Period).DurationStartAsDateObject;
                report.RPs128FEndDate = ((SBRDurationPeriod)sbrRPs128FContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPs128FElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPs128FContexts.Count(); i++)
                {
                    sbrRPs128FContext = sbrRPs128FContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPs128FContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPs128FElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPs128FElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPs128FElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPs128FElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.WithholdingTaxExemptNonResidentInterestPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS318 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPs128FContext
    
            #region sbrRPs128FAContext
            IEnumerable<SBRContext> sbrRPs128FAContexts = null;
            SBRContext sbrRPs128FAContext = null;
    
            sbrRPs128FAContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}WithholdingTaxExemptionDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Section128FA")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPs128FAElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPs128FAContexts != null && sbrRPs128FAContexts.Count() > 0)
            {
                sbrRPs128FAContext = sbrRPs128FAContexts.First();
        
                report.RPs128FAId = sbrRPs128FAContext.Id;
                report.RPs128FAIdentifier = sbrRPs128FAContext.EntityIdentifier;
                report.RPs128FAIdentifierScheme = sbrRPs128FAContext.EntityScheme;
        
                report.RPs128FACount = sbrRPs128FAContexts.Count();
                report.RPs128FAExists = true;
                report.RPs128FAOccurrenceIndex = Array.IndexOf(contextArray, sbrRPs128FAContext) + 1;
                report.RPs128FALastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPs128FAContexts.Last()) + 1;
        

                if (report.RPs128FAIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPs128FAIdentifierTFN = report.RPs128FAIdentifier;
    
                report.RPs128FAStartDate = ((SBRDurationPeriod)sbrRPs128FAContext.Period).DurationStartAsDateObject;
                report.RPs128FAEndDate = ((SBRDurationPeriod)sbrRPs128FAContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPs128FAElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPs128FAContexts.Count(); i++)
                {
                    sbrRPs128FAContext = sbrRPs128FAContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPs128FAContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPs128FAElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPs128FAElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPs128FAElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPs128FAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.WithholdingTaxExemptNonResidentInterestPaid.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS319 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPs128FAContext
    
            #region sbrRPAdvisoryRelatedPartyContext
            IEnumerable<SBRContext> sbrRPAdvisoryRelatedPartyContexts = null;
            SBRContext sbrRPAdvisoryRelatedPartyContext = null;
    
            sbrRPAdvisoryRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Advisory"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPAdvisoryRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPAdvisoryRelatedPartyContexts != null && sbrRPAdvisoryRelatedPartyContexts.Count() > 0)
            {
                sbrRPAdvisoryRelatedPartyContext = sbrRPAdvisoryRelatedPartyContexts.First();
        
                report.RPAdvisoryRelatedPartyId = sbrRPAdvisoryRelatedPartyContext.Id;
                report.RPAdvisoryRelatedPartyIdentifier = sbrRPAdvisoryRelatedPartyContext.EntityIdentifier;
                report.RPAdvisoryRelatedPartyIdentifierScheme = sbrRPAdvisoryRelatedPartyContext.EntityScheme;
        
                report.RPAdvisoryRelatedPartyCount = sbrRPAdvisoryRelatedPartyContexts.Count();
                report.RPAdvisoryRelatedPartyExists = true;
                report.RPAdvisoryRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAdvisoryRelatedPartyContext) + 1;
                report.RPAdvisoryRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPAdvisoryRelatedPartyContexts.Last()) + 1;
        

                if (report.RPAdvisoryRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPAdvisoryRelatedPartyIdentifierTFN = report.RPAdvisoryRelatedPartyIdentifier;
    
                report.RPAdvisoryRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPAdvisoryRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPAdvisoryRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPAdvisoryRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPAdvisoryRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPAdvisoryRelatedPartyContexts.Count(); i++)
                {
                    sbrRPAdvisoryRelatedPartyContext = sbrRPAdvisoryRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPAdvisoryRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPAdvisoryRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPAdvisoryRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPAdvisoryRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPAdvisoryRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS289 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPAdvisoryRelatedPartyContext
    
            #region sbrRPBorrowingRelatedPartyContext
            IEnumerable<SBRContext> sbrRPBorrowingRelatedPartyContexts = null;
            SBRContext sbrRPBorrowingRelatedPartyContext = null;
    
            sbrRPBorrowingRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Borrowing"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBorrowingRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBorrowingRelatedPartyContexts != null && sbrRPBorrowingRelatedPartyContexts.Count() > 0)
            {
                sbrRPBorrowingRelatedPartyContext = sbrRPBorrowingRelatedPartyContexts.First();
        
                report.RPBorrowingRelatedPartyId = sbrRPBorrowingRelatedPartyContext.Id;
                report.RPBorrowingRelatedPartyIdentifier = sbrRPBorrowingRelatedPartyContext.EntityIdentifier;
                report.RPBorrowingRelatedPartyIdentifierScheme = sbrRPBorrowingRelatedPartyContext.EntityScheme;
        
                report.RPBorrowingRelatedPartyCount = sbrRPBorrowingRelatedPartyContexts.Count();
                report.RPBorrowingRelatedPartyExists = true;
                report.RPBorrowingRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBorrowingRelatedPartyContext) + 1;
                report.RPBorrowingRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBorrowingRelatedPartyContexts.Last()) + 1;
        

                if (report.RPBorrowingRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPBorrowingRelatedPartyIdentifierTFN = report.RPBorrowingRelatedPartyIdentifier;
    
                report.RPBorrowingRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPBorrowingRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPBorrowingRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPBorrowingRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPBorrowingRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPBorrowingRelatedPartyContexts.Count(); i++)
                {
                    sbrRPBorrowingRelatedPartyContext = sbrRPBorrowingRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBorrowingRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBorrowingRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBorrowingRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBorrowingRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPBorrowingRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS271 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPBorrowingRelatedPartyContext
    
            #region sbrRPEligibleContractRelatedPartyContext
            IEnumerable<SBRContext> sbrRPEligibleContractRelatedPartyContexts = null;
            SBRContext sbrRPEligibleContractRelatedPartyContext = null;
    
            sbrRPEligibleContractRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}EligibleContract"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPEligibleContractRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPEligibleContractRelatedPartyContexts != null && sbrRPEligibleContractRelatedPartyContexts.Count() > 0)
            {
                sbrRPEligibleContractRelatedPartyContext = sbrRPEligibleContractRelatedPartyContexts.First();
        
                report.RPEligibleContractRelatedPartyId = sbrRPEligibleContractRelatedPartyContext.Id;
                report.RPEligibleContractRelatedPartyIdentifier = sbrRPEligibleContractRelatedPartyContext.EntityIdentifier;
                report.RPEligibleContractRelatedPartyIdentifierScheme = sbrRPEligibleContractRelatedPartyContext.EntityScheme;
        
                report.RPEligibleContractRelatedPartyCount = sbrRPEligibleContractRelatedPartyContexts.Count();
                report.RPEligibleContractRelatedPartyExists = true;
                report.RPEligibleContractRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPEligibleContractRelatedPartyContext) + 1;
                report.RPEligibleContractRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPEligibleContractRelatedPartyContexts.Last()) + 1;
        

                if (report.RPEligibleContractRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPEligibleContractRelatedPartyIdentifierTFN = report.RPEligibleContractRelatedPartyIdentifier;
    
                report.RPEligibleContractRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPEligibleContractRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPEligibleContractRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPEligibleContractRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPEligibleContractRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPEligibleContractRelatedPartyContexts.Count(); i++)
                {
                    sbrRPEligibleContractRelatedPartyContext = sbrRPEligibleContractRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPEligibleContractRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPEligibleContractRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPEligibleContractRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPEligibleContractRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPEligibleContractRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS283 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPEligibleContractRelatedPartyContext
    
            #region sbrRPGuaranteeTypeRelatedPartyContext
            IEnumerable<SBRContext> sbrRPGuaranteeTypeRelatedPartyContexts = null;
            SBRContext sbrRPGuaranteeTypeRelatedPartyContext = null;
    
            sbrRPGuaranteeTypeRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}GuaranteeType"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPGuaranteeTypeRelatedPartyContexts != null && sbrRPGuaranteeTypeRelatedPartyContexts.Count() > 0)
            {
                sbrRPGuaranteeTypeRelatedPartyContext = sbrRPGuaranteeTypeRelatedPartyContexts.First();
        
                report.RPGuaranteeTypeRelatedPartyId = sbrRPGuaranteeTypeRelatedPartyContext.Id;
                report.RPGuaranteeTypeRelatedPartyIdentifier = sbrRPGuaranteeTypeRelatedPartyContext.EntityIdentifier;
                report.RPGuaranteeTypeRelatedPartyIdentifierScheme = sbrRPGuaranteeTypeRelatedPartyContext.EntityScheme;
        
                report.RPGuaranteeTypeRelatedPartyCount = sbrRPGuaranteeTypeRelatedPartyContexts.Count();
                report.RPGuaranteeTypeRelatedPartyExists = true;
                report.RPGuaranteeTypeRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGuaranteeTypeRelatedPartyContext) + 1;
                report.RPGuaranteeTypeRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPGuaranteeTypeRelatedPartyContexts.Last()) + 1;
        

                if (report.RPGuaranteeTypeRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPGuaranteeTypeRelatedPartyIdentifierTFN = report.RPGuaranteeTypeRelatedPartyIdentifier;
    
                report.RPGuaranteeTypeRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPGuaranteeTypeRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPGuaranteeTypeRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPGuaranteeTypeRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPGuaranteeTypeRelatedPartyContexts.Count(); i++)
                {
                    sbrRPGuaranteeTypeRelatedPartyContext = sbrRPGuaranteeTypeRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPGuaranteeTypeRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPGuaranteeTypeRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS277 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPGuaranteeTypeRelatedPartyContext
    
            #region sbrRPHedgingRelatedPartyContext
            IEnumerable<SBRContext> sbrRPHedgingRelatedPartyContexts = null;
            SBRContext sbrRPHedgingRelatedPartyContext = null;
    
            sbrRPHedgingRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Hedging"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPHedgingRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPHedgingRelatedPartyContexts != null && sbrRPHedgingRelatedPartyContexts.Count() > 0)
            {
                sbrRPHedgingRelatedPartyContext = sbrRPHedgingRelatedPartyContexts.First();
        
                report.RPHedgingRelatedPartyId = sbrRPHedgingRelatedPartyContext.Id;
                report.RPHedgingRelatedPartyIdentifier = sbrRPHedgingRelatedPartyContext.EntityIdentifier;
                report.RPHedgingRelatedPartyIdentifierScheme = sbrRPHedgingRelatedPartyContext.EntityScheme;
        
                report.RPHedgingRelatedPartyCount = sbrRPHedgingRelatedPartyContexts.Count();
                report.RPHedgingRelatedPartyExists = true;
                report.RPHedgingRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHedgingRelatedPartyContext) + 1;
                report.RPHedgingRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHedgingRelatedPartyContexts.Last()) + 1;
        

                if (report.RPHedgingRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPHedgingRelatedPartyIdentifierTFN = report.RPHedgingRelatedPartyIdentifier;
    
                report.RPHedgingRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPHedgingRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPHedgingRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPHedgingRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPHedgingRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPHedgingRelatedPartyContexts.Count(); i++)
                {
                    sbrRPHedgingRelatedPartyContext = sbrRPHedgingRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPHedgingRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPHedgingRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPHedgingRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPHedgingRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPHedgingRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS292 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPHedgingRelatedPartyContext
    
            #region sbrRPInvestmentRelatedPartyContext
            IEnumerable<SBRContext> sbrRPInvestmentRelatedPartyContexts = null;
            SBRContext sbrRPInvestmentRelatedPartyContext = null;
    
            sbrRPInvestmentRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Investment"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPInvestmentRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPInvestmentRelatedPartyContexts != null && sbrRPInvestmentRelatedPartyContexts.Count() > 0)
            {
                sbrRPInvestmentRelatedPartyContext = sbrRPInvestmentRelatedPartyContexts.First();
        
                report.RPInvestmentRelatedPartyId = sbrRPInvestmentRelatedPartyContext.Id;
                report.RPInvestmentRelatedPartyIdentifier = sbrRPInvestmentRelatedPartyContext.EntityIdentifier;
                report.RPInvestmentRelatedPartyIdentifierScheme = sbrRPInvestmentRelatedPartyContext.EntityScheme;
        
                report.RPInvestmentRelatedPartyCount = sbrRPInvestmentRelatedPartyContexts.Count();
                report.RPInvestmentRelatedPartyExists = true;
                report.RPInvestmentRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestmentRelatedPartyContext) + 1;
                report.RPInvestmentRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPInvestmentRelatedPartyContexts.Last()) + 1;
        

                if (report.RPInvestmentRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPInvestmentRelatedPartyIdentifierTFN = report.RPInvestmentRelatedPartyIdentifier;
    
                report.RPInvestmentRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPInvestmentRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPInvestmentRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPInvestmentRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPInvestmentRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPInvestmentRelatedPartyContexts.Count(); i++)
                {
                    sbrRPInvestmentRelatedPartyContext = sbrRPInvestmentRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPInvestmentRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPInvestmentRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPInvestmentRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPInvestmentRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPInvestmentRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS286 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPInvestmentRelatedPartyContext
    
            #region sbrRPLendingRelatedPartyContext
            IEnumerable<SBRContext> sbrRPLendingRelatedPartyContexts = null;
            SBRContext sbrRPLendingRelatedPartyContext = null;
    
            sbrRPLendingRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Lending"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLendingRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLendingRelatedPartyContexts != null && sbrRPLendingRelatedPartyContexts.Count() > 0)
            {
                sbrRPLendingRelatedPartyContext = sbrRPLendingRelatedPartyContexts.First();
        
                report.RPLendingRelatedPartyId = sbrRPLendingRelatedPartyContext.Id;
                report.RPLendingRelatedPartyIdentifier = sbrRPLendingRelatedPartyContext.EntityIdentifier;
                report.RPLendingRelatedPartyIdentifierScheme = sbrRPLendingRelatedPartyContext.EntityScheme;
        
                report.RPLendingRelatedPartyCount = sbrRPLendingRelatedPartyContexts.Count();
                report.RPLendingRelatedPartyExists = true;
                report.RPLendingRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLendingRelatedPartyContext) + 1;
                report.RPLendingRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLendingRelatedPartyContexts.Last()) + 1;
        

                if (report.RPLendingRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLendingRelatedPartyIdentifierTFN = report.RPLendingRelatedPartyIdentifier;
    
                report.RPLendingRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPLendingRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPLendingRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPLendingRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLendingRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLendingRelatedPartyContexts.Count(); i++)
                {
                    sbrRPLendingRelatedPartyContext = sbrRPLendingRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLendingRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLendingRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLendingRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLendingRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLendingRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS274 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPLendingRelatedPartyContext
    
            #region sbrRPOtherRelatedPartyContext
            IEnumerable<SBRContext> sbrRPOtherRelatedPartyContexts = null;
            SBRContext sbrRPOtherRelatedPartyContext = null;
    
            sbrRPOtherRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Other"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherRelatedPartyContexts != null && sbrRPOtherRelatedPartyContexts.Count() > 0)
            {
                sbrRPOtherRelatedPartyContext = sbrRPOtherRelatedPartyContexts.First();
        
                report.RPOtherRelatedPartyId = sbrRPOtherRelatedPartyContext.Id;
                report.RPOtherRelatedPartyIdentifier = sbrRPOtherRelatedPartyContext.EntityIdentifier;
                report.RPOtherRelatedPartyIdentifierScheme = sbrRPOtherRelatedPartyContext.EntityScheme;
        
                report.RPOtherRelatedPartyCount = sbrRPOtherRelatedPartyContexts.Count();
                report.RPOtherRelatedPartyExists = true;
                report.RPOtherRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherRelatedPartyContext) + 1;
                report.RPOtherRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherRelatedPartyContexts.Last()) + 1;
        

                if (report.RPOtherRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherRelatedPartyIdentifierTFN = report.RPOtherRelatedPartyIdentifier;
    
                report.RPOtherRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPOtherRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPOtherRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPOtherRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherRelatedPartyContexts.Count(); i++)
                {
                    sbrRPOtherRelatedPartyContext = sbrRPOtherRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS295 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherRelatedPartyContext
    
            #region sbrRPTotalNetRelatedPartyContext
            IEnumerable<SBRContext> sbrRPTotalNetRelatedPartyContexts = null;
            SBRContext sbrRPTotalNetRelatedPartyContext = null;
    
            sbrRPTotalNetRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TotalNet"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTotalNetRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTotalNetRelatedPartyContexts != null && sbrRPTotalNetRelatedPartyContexts.Count() > 0)
            {
                sbrRPTotalNetRelatedPartyContext = sbrRPTotalNetRelatedPartyContexts.First();
        
                report.RPTotalNetRelatedPartyId = sbrRPTotalNetRelatedPartyContext.Id;
                report.RPTotalNetRelatedPartyIdentifier = sbrRPTotalNetRelatedPartyContext.EntityIdentifier;
                report.RPTotalNetRelatedPartyIdentifierScheme = sbrRPTotalNetRelatedPartyContext.EntityScheme;
        
                report.RPTotalNetRelatedPartyCount = sbrRPTotalNetRelatedPartyContexts.Count();
                report.RPTotalNetRelatedPartyExists = true;
                report.RPTotalNetRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTotalNetRelatedPartyContext) + 1;
                report.RPTotalNetRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTotalNetRelatedPartyContexts.Last()) + 1;
        

                if (report.RPTotalNetRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTotalNetRelatedPartyIdentifierTFN = report.RPTotalNetRelatedPartyIdentifier;
    
                report.RPTotalNetRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPTotalNetRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPTotalNetRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPTotalNetRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTotalNetRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTotalNetRelatedPartyContexts.Count(); i++)
                {
                    sbrRPTotalNetRelatedPartyContext = sbrRPTotalNetRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTotalNetRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTotalNetRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTotalNetRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTotalNetRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTotalNetRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS305 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPTotalNetRelatedPartyContext
    
            #region sbrRPTradingRelatedPartyContext
            IEnumerable<SBRContext> sbrRPTradingRelatedPartyContexts = null;
            SBRContext sbrRPTradingRelatedPartyContext = null;
    
            sbrRPTradingRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Trading"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTradingRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTradingRelatedPartyContexts != null && sbrRPTradingRelatedPartyContexts.Count() > 0)
            {
                sbrRPTradingRelatedPartyContext = sbrRPTradingRelatedPartyContexts.First();
        
                report.RPTradingRelatedPartyId = sbrRPTradingRelatedPartyContext.Id;
                report.RPTradingRelatedPartyIdentifier = sbrRPTradingRelatedPartyContext.EntityIdentifier;
                report.RPTradingRelatedPartyIdentifierScheme = sbrRPTradingRelatedPartyContext.EntityScheme;
        
                report.RPTradingRelatedPartyCount = sbrRPTradingRelatedPartyContexts.Count();
                report.RPTradingRelatedPartyExists = true;
                report.RPTradingRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTradingRelatedPartyContext) + 1;
                report.RPTradingRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTradingRelatedPartyContexts.Last()) + 1;
        

                if (report.RPTradingRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTradingRelatedPartyIdentifierTFN = report.RPTradingRelatedPartyIdentifier;
    
                report.RPTradingRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPTradingRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPTradingRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPTradingRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTradingRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTradingRelatedPartyContexts.Count(); i++)
                {
                    sbrRPTradingRelatedPartyContext = sbrRPTradingRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTradingRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTradingRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTradingRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTradingRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTradingRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS280 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPTradingRelatedPartyContext
    
            #region sbrRPLeasingRelatedPartyContext
            IEnumerable<SBRContext> sbrRPLeasingRelatedPartyContexts = null;
            SBRContext sbrRPLeasingRelatedPartyContext = null;
    
            sbrRPLeasingRelatedPartyContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}BankingTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}Leasing"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}CounterpartyRelationshipTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}RelatedParty")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPLeasingRelatedPartyElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPLeasingRelatedPartyContexts != null && sbrRPLeasingRelatedPartyContexts.Count() > 0)
            {
                sbrRPLeasingRelatedPartyContext = sbrRPLeasingRelatedPartyContexts.First();
        
                report.RPLeasingRelatedPartyId = sbrRPLeasingRelatedPartyContext.Id;
                report.RPLeasingRelatedPartyIdentifier = sbrRPLeasingRelatedPartyContext.EntityIdentifier;
                report.RPLeasingRelatedPartyIdentifierScheme = sbrRPLeasingRelatedPartyContext.EntityScheme;
        
                report.RPLeasingRelatedPartyCount = sbrRPLeasingRelatedPartyContexts.Count();
                report.RPLeasingRelatedPartyExists = true;
                report.RPLeasingRelatedPartyOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLeasingRelatedPartyContext) + 1;
                report.RPLeasingRelatedPartyLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPLeasingRelatedPartyContexts.Last()) + 1;
        

                if (report.RPLeasingRelatedPartyIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPLeasingRelatedPartyIdentifierTFN = report.RPLeasingRelatedPartyIdentifier;
    
                report.RPLeasingRelatedPartyStartDate = ((SBRDurationPeriod)sbrRPLeasingRelatedPartyContext.Period).DurationStartAsDateObject;
                report.RPLeasingRelatedPartyEndDate = ((SBRDurationPeriod)sbrRPLeasingRelatedPartyContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPLeasingRelatedPartyElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPLeasingRelatedPartyContexts.Count(); i++)
                {
                    sbrRPLeasingRelatedPartyContext = sbrRPLeasingRelatedPartyContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPLeasingRelatedPartyContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPLeasingRelatedPartyElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPLeasingRelatedPartyElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPLeasingRelatedPartyElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPLeasingRelatedPartyElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}FinancialServices.OffshoreBankingIncomeAssessable.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS328 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPLeasingRelatedPartyContext
    
            #region sbrRPBranchOperationsInternallyRecordedDealingsContext
            IEnumerable<SBRContext> sbrRPBranchOperationsInternallyRecordedDealingsContexts = null;
            SBRContext sbrRPBranchOperationsInternallyRecordedDealingsContext = null;
    
            sbrRPBranchOperationsInternallyRecordedDealingsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}BranchOperations"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.TYPED && dd.Dimension.EndsWith("}InternallyRecordedDealingsDimension")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBranchOperationsInternallyRecordedDealingsContexts != null && sbrRPBranchOperationsInternallyRecordedDealingsContexts.Count() > 0)
            {
        
                report.RPBranchOperationsInternallyRecordedDealingsContextCollection = new List<IDS2021.RPBranchOperationsInternallyRecordedDealingsContext>();
                for (int i = 0; i < sbrRPBranchOperationsInternallyRecordedDealingsContexts.Count(); i++)
                {
                    sbrRPBranchOperationsInternallyRecordedDealingsContext = sbrRPBranchOperationsInternallyRecordedDealingsContexts.ElementAt(i);
                    IDS2021.RPBranchOperationsInternallyRecordedDealingsContext rpBranchOperationsInternallyRecordedDealingsContext = new IDS2021.RPBranchOperationsInternallyRecordedDealingsContext();
                    report.RPBranchOperationsInternallyRecordedDealingsContextCollection.Add(rpBranchOperationsInternallyRecordedDealingsContext);
            
                    rpBranchOperationsInternallyRecordedDealingsContext.Id = sbrRPBranchOperationsInternallyRecordedDealingsContext.Id;
                    rpBranchOperationsInternallyRecordedDealingsContext.Identifier = sbrRPBranchOperationsInternallyRecordedDealingsContext.EntityIdentifier;
                    rpBranchOperationsInternallyRecordedDealingsContext.IdentifierScheme = sbrRPBranchOperationsInternallyRecordedDealingsContext.EntityScheme;
            
                    rpBranchOperationsInternallyRecordedDealingsContext.Count = sbrRPBranchOperationsInternallyRecordedDealingsContexts.Count();
                    rpBranchOperationsInternallyRecordedDealingsContext.Exists = true;
                    rpBranchOperationsInternallyRecordedDealingsContext.OccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInternallyRecordedDealingsContext) + 1;
                    rpBranchOperationsInternallyRecordedDealingsContext.LastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInternallyRecordedDealingsContexts.Last()) + 1;
            
    
                    if (rpBranchOperationsInternallyRecordedDealingsContext.IdentifierScheme == "http://www.ato.gov.au/tfn") rpBranchOperationsInternallyRecordedDealingsContext.IdentifierTFN = rpBranchOperationsInternallyRecordedDealingsContext.Identifier;
        
                    rpBranchOperationsInternallyRecordedDealingsContext.StartDate = ((SBRDurationPeriod)sbrRPBranchOperationsInternallyRecordedDealingsContext.Period).DurationStartAsDateObject;
                    rpBranchOperationsInternallyRecordedDealingsContext.EndDate = ((SBRDurationPeriod)sbrRPBranchOperationsInternallyRecordedDealingsContext.Period).DurationEndAsDateObject;
            
                    currentSegment = sbrRPBranchOperationsInternallyRecordedDealingsContext.Segment.Segment;
            
                    currentNamespaceManager = new XmlNamespaceManager(currentSegment.OwnerDocument.NameTable);
                    currentNamespaceManager.AddNamespace("xbrldi", "http://xbrl.org/2006/xbrldi");
            
                    currentNode = currentSegment.SelectSingleNode("xbrldi:typedMember[contains(@dimension, 'InternallyRecordedDealingsDimension')]", currentNamespaceManager);
            
                    if (currentNode != null)
                    {
                        rpBranchOperationsInternallyRecordedDealingsContext.InternallyRecordedDealingsDimensionValue = currentNode.InnerText.Trim();
                    }
            
                    sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBranchOperationsInternallyRecordedDealingsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
                    rpBranchOperationsInternallyRecordedDealingsContext.IDS171 = rpBranchOperationsInternallyRecordedDealingsContext.InternallyRecordedDealingsDimensionValue;
            
                    if (sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.InternallyRecordedDealingsAmountsClaimedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpBranchOperationsInternallyRecordedDealingsContext.IDS169 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsInternallyRecordedDealingsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.InternallyRecordedDealingsAmountsReturnedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) rpBranchOperationsInternallyRecordedDealingsContext.IDS170 = decimal.Parse(currentValues[0].Value);
                    }
        } 
            } 
            #endregion End of Context sbrRPBranchOperationsInternallyRecordedDealingsContext
    
            #region sbrRPBranchOperationsInterestFreeContext
            IEnumerable<SBRContext> sbrRPBranchOperationsInterestFreeContexts = null;
            SBRContext sbrRPBranchOperationsInterestFreeContext = null;
    
            sbrRPBranchOperationsInterestFreeContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}BranchOperations"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}LoanTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}InterestFree")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBranchOperationsInterestFreeElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBranchOperationsInterestFreeContexts != null && sbrRPBranchOperationsInterestFreeContexts.Count() > 0)
            {
                sbrRPBranchOperationsInterestFreeContext = sbrRPBranchOperationsInterestFreeContexts.First();
        
                report.RPBranchOperationsInterestFreeId = sbrRPBranchOperationsInterestFreeContext.Id;
                report.RPBranchOperationsInterestFreeIdentifier = sbrRPBranchOperationsInterestFreeContext.EntityIdentifier;
                report.RPBranchOperationsInterestFreeIdentifierScheme = sbrRPBranchOperationsInterestFreeContext.EntityScheme;
        
                report.RPBranchOperationsInterestFreeCount = sbrRPBranchOperationsInterestFreeContexts.Count();
                report.RPBranchOperationsInterestFreeExists = true;
                report.RPBranchOperationsInterestFreeOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInterestFreeContext) + 1;
                report.RPBranchOperationsInterestFreeLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInterestFreeContexts.Last()) + 1;
        

                if (report.RPBranchOperationsInterestFreeIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPBranchOperationsInterestFreeIdentifierTFN = report.RPBranchOperationsInterestFreeIdentifier;
    
                report.RPBranchOperationsInterestFreeStartDate = ((SBRDurationPeriod)sbrRPBranchOperationsInterestFreeContext.Period).DurationStartAsDateObject;
                report.RPBranchOperationsInterestFreeEndDate = ((SBRDurationPeriod)sbrRPBranchOperationsInterestFreeContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPBranchOperationsInterestFreeElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPBranchOperationsInterestFreeContexts.Count(); i++)
                {
                    sbrRPBranchOperationsInterestFreeContext = sbrRPBranchOperationsInterestFreeContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBranchOperationsInterestFreeContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBranchOperationsInterestFreeElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBranchOperationsInterestFreeElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBranchOperationsInterestFreeElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestFreeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.BorrowedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS165 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestFreeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.LoanedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS166 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPBranchOperationsInterestFreeContext
    
            #region sbrRPBranchOperationsInterestBearingContext
            IEnumerable<SBRContext> sbrRPBranchOperationsInterestBearingContexts = null;
            SBRContext sbrRPBranchOperationsInterestBearingContext = null;
    
            sbrRPBranchOperationsInterestBearingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}BranchOperations"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}LoanTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}InterestBearing")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPBranchOperationsInterestBearingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPBranchOperationsInterestBearingContexts != null && sbrRPBranchOperationsInterestBearingContexts.Count() > 0)
            {
                sbrRPBranchOperationsInterestBearingContext = sbrRPBranchOperationsInterestBearingContexts.First();
        
                report.RPBranchOperationsInterestBearingId = sbrRPBranchOperationsInterestBearingContext.Id;
                report.RPBranchOperationsInterestBearingIdentifier = sbrRPBranchOperationsInterestBearingContext.EntityIdentifier;
                report.RPBranchOperationsInterestBearingIdentifierScheme = sbrRPBranchOperationsInterestBearingContext.EntityScheme;
        
                report.RPBranchOperationsInterestBearingCount = sbrRPBranchOperationsInterestBearingContexts.Count();
                report.RPBranchOperationsInterestBearingExists = true;
                report.RPBranchOperationsInterestBearingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInterestBearingContext) + 1;
                report.RPBranchOperationsInterestBearingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPBranchOperationsInterestBearingContexts.Last()) + 1;
        

                if (report.RPBranchOperationsInterestBearingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPBranchOperationsInterestBearingIdentifierTFN = report.RPBranchOperationsInterestBearingIdentifier;
    
                report.RPBranchOperationsInterestBearingStartDate = ((SBRDurationPeriod)sbrRPBranchOperationsInterestBearingContext.Period).DurationStartAsDateObject;
                report.RPBranchOperationsInterestBearingEndDate = ((SBRDurationPeriod)sbrRPBranchOperationsInterestBearingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPBranchOperationsInterestBearingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPBranchOperationsInterestBearingContexts.Count(); i++)
                {
                    sbrRPBranchOperationsInterestBearingContext = sbrRPBranchOperationsInterestBearingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPBranchOperationsInterestBearingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPBranchOperationsInterestBearingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPBranchOperationsInterestBearingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPBranchOperationsInterestBearingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.BorrowedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS161 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.LoanedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS162 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.InterestBorrowedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS163 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPBranchOperationsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Income.InterestLoanedTotal.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS164 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPBranchOperationsInterestBearingContext
    
            #region sbrRPOtherFinancialDealingsInterestFreeContext
            IEnumerable<SBRContext> sbrRPOtherFinancialDealingsInterestFreeContexts = null;
            SBRContext sbrRPOtherFinancialDealingsInterestFreeContext = null;
    
            sbrRPOtherFinancialDealingsInterestFreeContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherFinancialDealings"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}LoanTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}InterestFree")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherFinancialDealingsInterestFreeContexts != null && sbrRPOtherFinancialDealingsInterestFreeContexts.Count() > 0)
            {
                sbrRPOtherFinancialDealingsInterestFreeContext = sbrRPOtherFinancialDealingsInterestFreeContexts.First();
        
                report.RPOtherFinancialDealingsInterestFreeId = sbrRPOtherFinancialDealingsInterestFreeContext.Id;
                report.RPOtherFinancialDealingsInterestFreeIdentifier = sbrRPOtherFinancialDealingsInterestFreeContext.EntityIdentifier;
                report.RPOtherFinancialDealingsInterestFreeIdentifierScheme = sbrRPOtherFinancialDealingsInterestFreeContext.EntityScheme;
        
                report.RPOtherFinancialDealingsInterestFreeCount = sbrRPOtherFinancialDealingsInterestFreeContexts.Count();
                report.RPOtherFinancialDealingsInterestFreeExists = true;
                report.RPOtherFinancialDealingsInterestFreeOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsInterestFreeContext) + 1;
                report.RPOtherFinancialDealingsInterestFreeLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsInterestFreeContexts.Last()) + 1;
        

                if (report.RPOtherFinancialDealingsInterestFreeIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherFinancialDealingsInterestFreeIdentifierTFN = report.RPOtherFinancialDealingsInterestFreeIdentifier;
    
                report.RPOtherFinancialDealingsInterestFreeStartDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsInterestFreeContext.Period).DurationStartAsDateObject;
                report.RPOtherFinancialDealingsInterestFreeEndDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsInterestFreeContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherFinancialDealingsInterestFreeContexts.Count(); i++)
                {
                    sbrRPOtherFinancialDealingsInterestFreeContext = sbrRPOtherFinancialDealingsInterestFreeContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherFinancialDealingsInterestFreeContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.BorrowedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS99 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsInterestFreeElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.LoanedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS101 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherFinancialDealingsInterestFreeContext
    
            #region sbrRPOtherFinancialDealingsInterestBearingContext
            IEnumerable<SBRContext> sbrRPOtherFinancialDealingsInterestBearingContexts = null;
            SBRContext sbrRPOtherFinancialDealingsInterestBearingContext = null;
    
            sbrRPOtherFinancialDealingsInterestBearingContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 3 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InternationalRelatedPartyDealingsTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}OtherFinancialDealings"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}LoanTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}InterestBearing")) )))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPOtherFinancialDealingsInterestBearingContexts != null && sbrRPOtherFinancialDealingsInterestBearingContexts.Count() > 0)
            {
                sbrRPOtherFinancialDealingsInterestBearingContext = sbrRPOtherFinancialDealingsInterestBearingContexts.First();
        
                report.RPOtherFinancialDealingsInterestBearingId = sbrRPOtherFinancialDealingsInterestBearingContext.Id;
                report.RPOtherFinancialDealingsInterestBearingIdentifier = sbrRPOtherFinancialDealingsInterestBearingContext.EntityIdentifier;
                report.RPOtherFinancialDealingsInterestBearingIdentifierScheme = sbrRPOtherFinancialDealingsInterestBearingContext.EntityScheme;
        
                report.RPOtherFinancialDealingsInterestBearingCount = sbrRPOtherFinancialDealingsInterestBearingContexts.Count();
                report.RPOtherFinancialDealingsInterestBearingExists = true;
                report.RPOtherFinancialDealingsInterestBearingOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsInterestBearingContext) + 1;
                report.RPOtherFinancialDealingsInterestBearingLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPOtherFinancialDealingsInterestBearingContexts.Last()) + 1;
        

                if (report.RPOtherFinancialDealingsInterestBearingIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPOtherFinancialDealingsInterestBearingIdentifierTFN = report.RPOtherFinancialDealingsInterestBearingIdentifier;
    
                report.RPOtherFinancialDealingsInterestBearingStartDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsInterestBearingContext.Period).DurationStartAsDateObject;
                report.RPOtherFinancialDealingsInterestBearingEndDate = ((SBRDurationPeriod)sbrRPOtherFinancialDealingsInterestBearingContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPOtherFinancialDealingsInterestBearingContexts.Count(); i++)
                {
                    sbrRPOtherFinancialDealingsInterestBearingContext = sbrRPOtherFinancialDealingsInterestBearingContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPOtherFinancialDealingsInterestBearingContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Liabilities.BorrowedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS98 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPOtherFinancialDealingsInterestBearingElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Assets.LoanedAverageBalance.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS100 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPOtherFinancialDealingsInterestBearingContext
    
            #region sbrRPForeignHubsContext
            IEnumerable<SBRContext> sbrRPForeignHubsContexts = null;
            SBRContext sbrRPForeignHubsContext = null;
    
            sbrRPForeignHubsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}InterestForeignEntitiesTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ForeignHubs")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPForeignHubsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPForeignHubsContexts != null && sbrRPForeignHubsContexts.Count() > 0)
            {
                sbrRPForeignHubsContext = sbrRPForeignHubsContexts.First();
        
                report.RPForeignHubsId = sbrRPForeignHubsContext.Id;
                report.RPForeignHubsIdentifier = sbrRPForeignHubsContext.EntityIdentifier;
                report.RPForeignHubsIdentifierScheme = sbrRPForeignHubsContext.EntityScheme;
        
                report.RPForeignHubsCount = sbrRPForeignHubsContexts.Count();
                report.RPForeignHubsExists = true;
                report.RPForeignHubsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPForeignHubsContext) + 1;
                report.RPForeignHubsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPForeignHubsContexts.Last()) + 1;
        

                if (report.RPForeignHubsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPForeignHubsIdentifierTFN = report.RPForeignHubsIdentifier;
    
                report.RPForeignHubsStartDate = ((SBRDurationPeriod)sbrRPForeignHubsContext.Period).DurationStartAsDateObject;
                report.RPForeignHubsEndDate = ((SBRDurationPeriod)sbrRPForeignHubsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPForeignHubsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPForeignHubsContexts.Count(); i++)
                {
                    sbrRPForeignHubsContext = sbrRPForeignHubsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPForeignHubsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPForeignHubsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPForeignHubsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPForeignHubsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
        
                    IEnumerable<SBRElement> sbrHubArrangementCollection;
                    if (populatedReportElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}HubArrangement", out currentValue))
                        sbrHubArrangementCollection = currentValue.Occurrences;
                    else
                        sbrHubArrangementCollection = null;
            
                    #region sbrHubArrangementCollection
            
                    if (sbrHubArrangementCollection != null)
                    {
                        report.HubArrangementCollectionExists = true;
                        report.HubArrangementCollectionCount = sbrHubArrangementCollection.Count();
                        if (sbrHubArrangementCollection != null && sbrHubArrangementCollection.Count() > 0)
                        {
                            report.HubArrangementCollection = new List<IDS2021.HubArrangement>();
                            for (int tupleIndex = 0; tupleIndex < sbrHubArrangementCollection.Count(); tupleIndex++)
                            {
                                SBRElement sbrHubArrangement;
                                sbrHubArrangement = sbrHubArrangementCollection.ElementAt(tupleIndex);
                
                                IDictionary<string, SBRElement> sbrHubArrangementTupleElementMap;
                                sbrHubArrangementTupleElementMap = sbrHubArrangement.PopulatedChildElementsMap ?? new Dictionary<string, SBRElement>();
                
                                IDS2021.HubArrangement hubArrangement = new IDS2021.HubArrangement();
                                report.HubArrangementCollection.Add(hubArrangement);
                                hubArrangement.OccurrenceIndex = tupleIndex + 1;
                        
                                if (sbrHubArrangementTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HubArrangement.Code", out currentValue))
                                {
                                    if (!currentValue.IsNil) hubArrangement.IDS394 = currentValue.Value;
                                }
                        
                                if (sbrHubArrangementTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Expense.HubArrangementImports.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) hubArrangement.IDS395 = decimal.Parse(currentValue.Value);
                                }
                        
                                if (sbrHubArrangementTupleElementMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Revenue.HubArrangementExports.Amount", out currentValue))
                                {
                                    if (!currentValue.IsNil) hubArrangement.IDS396 = decimal.Parse(currentValue.Value);
                                }
                    } 
                    #endregion End of Tuple sbrHubArrangementCollection
                        } 
                    } 
                } 
            } 
            #endregion End of Context sbrRPForeignHubsContext
    
            #region sbrRPHybridInstrumentsContext
            IEnumerable<SBRContext> sbrRPHybridInstrumentsContexts = null;
            SBRContext sbrRPHybridInstrumentsContext = null;
    
            sbrRPHybridInstrumentsContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}FinancialArrangementTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}HybridInstruments")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPHybridInstrumentsElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPHybridInstrumentsContexts != null && sbrRPHybridInstrumentsContexts.Count() > 0)
            {
                sbrRPHybridInstrumentsContext = sbrRPHybridInstrumentsContexts.First();
        
                report.RPHybridInstrumentsId = sbrRPHybridInstrumentsContext.Id;
                report.RPHybridInstrumentsIdentifier = sbrRPHybridInstrumentsContext.EntityIdentifier;
                report.RPHybridInstrumentsIdentifierScheme = sbrRPHybridInstrumentsContext.EntityScheme;
        
                report.RPHybridInstrumentsCount = sbrRPHybridInstrumentsContexts.Count();
                report.RPHybridInstrumentsExists = true;
                report.RPHybridInstrumentsOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHybridInstrumentsContext) + 1;
                report.RPHybridInstrumentsLastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPHybridInstrumentsContexts.Last()) + 1;
        

                if (report.RPHybridInstrumentsIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPHybridInstrumentsIdentifierTFN = report.RPHybridInstrumentsIdentifier;
    
                report.RPHybridInstrumentsStartDate = ((SBRDurationPeriod)sbrRPHybridInstrumentsContext.Period).DurationStartAsDateObject;
                report.RPHybridInstrumentsEndDate = ((SBRDurationPeriod)sbrRPHybridInstrumentsContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPHybridInstrumentsElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPHybridInstrumentsContexts.Count(); i++)
                {
                    sbrRPHybridInstrumentsContext = sbrRPHybridInstrumentsContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPHybridInstrumentsContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPHybridInstrumentsElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPHybridInstrumentsElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPHybridInstrumentsElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPHybridInstrumentsElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}Miscellaneous.HybridMismatchDeductions.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS483 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPHybridInstrumentsContext
    
            #region sbrRPTOFAContext
            IEnumerable<SBRContext> sbrRPTOFAContexts = null;
            SBRContext sbrRPTOFAContext = null;
    
            sbrRPTOFAContexts = contextArray.Where(
                c => c.Period != null && c.Period.PeriodType == SBRPeriod_PeriodType.DURATION &&
                c.Segment != null && c.Segment.ExplicitOrTypedDimensionDomains.Count == 2 && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}ReportPartyTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}ReportingParty"))  && 
                (c.Segment.ExplicitOrTypedDimensionDomains.Any(dd => dd.DimensionType == SBRExplicitOrTypedDimensionDomain_Type.EXPLICIT && dd.Dimension.EndsWith("}FinancialArrangementTypeDimension") && ((SBRExplicitDimensionDomain)dd).Domain.EndsWith("}TOFA")) ))).ToList();
            Dictionary<string, IList<SBRElement>> sbrPopulatedRPTOFAElementsMap = new Dictionary<string, IList<SBRElement>>();
            if (sbrRPTOFAContexts != null && sbrRPTOFAContexts.Count() > 0)
            {
                sbrRPTOFAContext = sbrRPTOFAContexts.First();
        
                report.RPTOFAId = sbrRPTOFAContext.Id;
                report.RPTOFAIdentifier = sbrRPTOFAContext.EntityIdentifier;
                report.RPTOFAIdentifierScheme = sbrRPTOFAContext.EntityScheme;
        
                report.RPTOFACount = sbrRPTOFAContexts.Count();
                report.RPTOFAExists = true;
                report.RPTOFAOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTOFAContext) + 1;
                report.RPTOFALastOccurrenceIndex = Array.IndexOf(contextArray, sbrRPTOFAContexts.Last()) + 1;
        

                if (report.RPTOFAIdentifierScheme == "http://www.ato.gov.au/tfn") report.RPTOFAIdentifierTFN = report.RPTOFAIdentifier;
    
                report.RPTOFAStartDate = ((SBRDurationPeriod)sbrRPTOFAContext.Period).DurationStartAsDateObject;
                report.RPTOFAEndDate = ((SBRDurationPeriod)sbrRPTOFAContext.Period).DurationEndAsDateObject;
        
                sbrPopulatedRPTOFAElementsMap = new System.Collections.Generic.Dictionary<string, IList<SBRElement>>();
                for (int i = 0; i < sbrRPTOFAContexts.Count(); i++)
                {
                    sbrRPTOFAContext = sbrRPTOFAContexts.ElementAt(i);
            
                    IList<SBRElement> elements = sbrDocument.ElementsByContextId[sbrRPTOFAContext.Id];
                    if (elements != null)
                    {
                        for (int e = 0; e < elements.Count(); e++)
                        {
                            SBRElement currentElement = elements[e];
                            if (currentElement == null || currentElement.IsInTuple) continue; // Don't add tuple elements here, they will be found through a different map.
                            if (sbrPopulatedRPTOFAElementsMap.ContainsKey("{" + currentElement.Namespace + "}" + currentElement.Name))
                            {
                                sbrPopulatedRPTOFAElementsMap["{" + currentElement.Namespace + "}" + currentElement.Name].Add(currentElement);
                            }
                            else
                            {
                                sbrPopulatedRPTOFAElementsMap.Add("{" + currentElement.Namespace + "}" + currentElement.Name, new List<SBRElement> { currentElement } );
                            }
                        }
                    }
            
                    if (sbrPopulatedRPTOFAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxationOfFinancialArrangementsTotalValue.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS430 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTOFAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxationOfFinancialArrangementsTotalGains.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS431 = decimal.Parse(currentValues[0].Value);
                    }
            
                    if (sbrPopulatedRPTOFAElementsMap.TryGetValue("{http://www.sbr.gov.au/ato/ids}InternationalDealings.TaxationOfFinancialArrangementsTotalLosses.Amount", out currentValues))
                    {
                        if (!currentValues[0].IsNil) report.IDS432 = decimal.Parse(currentValues[0].Value);
                    }
                } 
            } 
            #endregion End of Context sbrRPTOFAContext
            return report;

        } // Of Consume Method
    } // Of Class
} // Of Namespace

